import logging, shutil

from aksy.devices.akai import sysex_types, model
from aksy.devices.akai.sampler import Sampler
from aksy.devices.akai.z48.sampler import Z48
from aksyx import AkaiSampler
from aksy import fileutils

log = logging.getLogger('aksy')

class MockConnector(object):
    def __init__(self):
        self.sample_file = ''
        self.program_file = ''
        
    def get(self, filename, destpath=None, source=AkaiSampler.MEMORY):
        log.debug("Transferring file %s to host from source %i" % (filename, source))
        try:
            if fileutils.is_sample(filename):
                shutil.copy(self.sample_file, destpath)
            elif fileutils.is_program(filename):
                shutil.copy(self.program_file, destpath)
        except IOError:
            raise model.SamplerException("Failed to get ", filename)

    def put(self, path, remote_name=None, destination=AkaiSampler.MEMORY):
        log.debug("Transferring file %s to sampler at remote_name %s (%i)" 
                  % (path, remote_name, destination))
    
    def execute(self, command, args, request_id=0):
        log.debug("execute(%s, %s)" % (repr(command), repr(args),))
        return None
    
    def close(self):
        log.debug("close()")
        
    
class MockZ48(Z48):
    def __init__(self, connector, debug=1): 
        # TODO: enable call to super class c'tor
        Sampler.__init__(self, connector)
        self.connector = connector
        self.debug = debug
        self.setup_tools()

        self._patch_disktools_get_disklist()
        
        self._patch_systemtools()
        
        self.programtools.get_handles_names = lambda : (1, 'program', 2, 'program 2')
        self.songtools.get_handles_names = lambda : (1, 'song',)
        self.multitools.get_handles_names = lambda : (1, 'multi', 2, 'multi 2')
        self.sampletools.get_handles_names = lambda : (1, 'sample', 2, 'sample 2')
        self.recordingtools.get_name = lambda : 'sample'
        
        self.setup_model()

        self._populate_fs()

        self._patch_rootdisk_getdir()

    def set_sample(self, sample):
	self.connector.sample_file = sample

    def set_program(self, program):
	self.connector.program_file = program

    def _populate_fs(self):
        mellotron_folder = model.Folder('Mellotron Samples')
        choir_folder = model.Folder('Choir')
        choir_folder.children.extend(
            (model.FileRef('Mellotron/Choir/Choir.AKM', 7102),
            model.FileRef('Mellotron/Choir/Choir.AKP', 7707),
            model.FileRef('Mellotron/Choir/Vox1.wav'),))

        mellotron_folder.children.extend(
            (choir_folder,
            model.FileRef('Mellotron Samples/A Sample.AKP'),
            model.FileRef('Mellotron Samples/Sample.wav'),))
        first_disk = self.disks.get_children()[0] 
        first_disk.root.children = [model.Folder('Autoload'),
             model.Folder('Songs')]
        self.disks.get_children()[1].root.children = [mellotron_folder]

        memory_items = [model.Sample("Boo", 1),
            model.Multi("Default", 2),]
        for i in range(0, 100):
            memory_items.append(model.Sample("Sample%i" %i, i))
        self.memory.set_children(memory_items)
        self.memory.get_children()[0].get_modified = lambda : True

    def _patch_disktools_get_disklist(self):
        def get_disklist(): 
            return [(256, 1, 0, 3, True, "Samples disk"),
                    (512, 1, 0, 3, False, "Cdrom")]
        
        self.disktools.get_disklist = get_disklist

    def _patch_systemtools(self):
        self.systemtools.get_free_wave_mem_size = lambda : 4
        self.systemtools.get_wave_mem_size = lambda : 16
        
    def _patch_rootdisk_getdir(self):
        def get_subdir(obj, path):
            for child in obj.get_children():
                if child.get_name() == path:
                    return child

        def get_dir(path):
            segments = path.split('/')
            folder = self.disks
            while segments and folder is not None:
                folder = get_subdir(folder, segments.pop(0))
            return folder

        self.disks.get_dir = get_dir
        
    def close(self):
        pass




""" Transfertools

Methods to send and receive files from the sampler.
"""

__author__ =  'Walco van Loon'
__version__=  "$Rev: 1354 $"

import os.path, logging

import aksy.devices.akai.sysex
from aksyx import AkaiSampler

LOG = logging.getLogger('aksy.devices.akai.transfertools')

class Transfertools:
    def __init__(self, connector):
        self.connector = connector
        self.get_cmd = aksy.devices.akai.sysex.Command('', '', 'transfertools', 'get', (aksy.devices.akai.sysex_types.STRING, aksy.devices.akai.sysex_types.STRING, aksy.devices.akai.sysex_types.STRING), None)
        self.put_cmd = aksy.devices.akai.sysex.Command('', '', 'transfertools', 'put', (aksy.devices.akai.sysex_types.STRING, aksy.devices.akai.sysex_types.STRING, aksy.devices.akai.sysex_types.STRING), None)

    def get(self, filename, destfile=None, source=AkaiSampler.MEMORY):
        """Gets a file from the sampler, overwriting destfile if it already exists.
        """
        if LOG.isEnabledFor(logging.DEBUG):
            LOG.debug("get(%s, %s, %i)", filename, destfile, source)

        if destfile is None:
            destfile = filename

        if hasattr(self.connector, 'get'):
            return self.connector.get(filename, destfile, source)
        else:
            return self.connector.execute(self.get_cmd, (filename, destfile, source))

    def put(self, sourcepath, remote_name=None, destination=AkaiSampler.MEMORY):
        """Transfers a file to the sampler, overwriting it if it already exists.
        Default destination is memory
        """
        if LOG.isEnabledFor(logging.DEBUG):
            LOG.debug("put(%s, %s, %i)", sourcepath, remote_name, destination)

        if remote_name is None:
            remote_name = os.path.basename(sourcepath)

        if hasattr(self.connector, 'put'):
            return self.connector.put(sourcepath, remote_name, destination)
        else:
            return self.connector.execute(self.put_cmd, (sourcepath, remote_name, destination,))



import gtk

import base, editors, keygrouppanel, zonepanel, modmatrix, filterpanel, lfopanel
import keygroupenvelope

from aksui.ak import keygroup

class KeygroupEditorWindow(gtk.Window):
    def __init__(self, s, p):
        gtk.Window.__init__(self)
        self.connect("delete_event", self.delete_event)
        self.set_position(gtk.WIN_POS_CENTER) 
        self.s = s
        self.setup(p)

    def delete_event(self, widget, event, data=None):
        self.hide()
        return True

    def clear_children(self):
        for i in (self.get_children()):
            self.remove(i)

    def setup(self, p):
        self.clear_children()
        self.p = p
        self.set_default_size(1000,600)
        #TODO: Don't completely rebuild this every time program changes.
        self.editor = KeygroupEditor(self.s, p)
        self.add(self.editor.editor)
        self.update_title()

    def update_title(self):
        self.set_title("aksui: %s" % (self.p.name))
        
class KeygroupEditor(base.Base):
    def __init__(self, s, p):
        base.Base.__init__(self, p, "keygroupEditorZ")
        self.s = p.s
        self.p = p
        if self.p.type == 1:
            self.keygroupEditor = editors.DrumEditorTable(s,p)
        else:
            self.keygroupEditor = editors.KeygroupEditorVBox(s,p)
            
        self.keygroupEditor.on_toggled_callback = self.on_toggled_callback
        
        # get first keygroup
        self.curr_keygroup = keygroup.get_keygroup_cached(p,0)
        self.panels = []
        
        self.rightVBox = gtk.VBox()

        panel_list = [
                     ("keygroupPanel", keygrouppanel.KeygroupPanel),
                     ("zonePanel", zonepanel.ZonePanel),
                     ("filterPanel", filterpanel.FilterPanel),
                     ("LFOPanel", lfopanel.LFOPanel),
                     ("keygroupEnvelopes", keygroupenvelope.KeygroupEnvelopes),
                     ("modMatrix", modmatrix.ModMatrix),
                    ] 
       
        for (n, ui_type) in panel_list:
            panel = ui_type(self.curr_keygroup, self.update_status)
            setattr(self, n, panel)
            self.panels.append(panel)

        for panel in self.panels:
            self.rightVBox.pack_start(panel, False, False, 0)
        
        self.w_entryProgramName.set_text(p.name)
        self.w_viewportKeygroups.add(self.keygroupEditor)
        self.w_viewportSlats.add(self.rightVBox)

        rbg = None
        curr_mode = self.curr_keygroup.gettools().get_edit_mode()
        modes = ["ONE","ALL","ADD"]
        i = 0
        for mode in modes:
            tb = gtk.RadioButton(rbg, mode)
            if not rbg:
                rbg = tb
            self.w_hboxNameEdit.add(tb)
            tb.connect("toggled", self.on_button_press_event, (i))
            if curr_mode == i:
                tb.set_active(True)
            i = i + 1

        children = self.w_statusbar.get_children()[0].get_children()
        self.statuslabel = children[0]
        
    def on_zone_sample_changed(self, widget, zone_combo, zone_label):
        text = zone_combo.get_text()
        #print zone_label
        zone_label.set_text(text)

    def on_button_press_event(self, widget, mode):
        modes = ["ONE","ALL","ADD"]
        self.curr_keygroup.gettools().set_edit_mode(mode)
        
    def change_keygroup(self, keygroup_index):
        self.curr_keygroup = keygroup.get_keygroup_cached(self.p,keygroup_index)
        self.curr_keygroup.set_current()
        for panel in self.panels:
            panel.setup(self.curr_keygroup)
            
        if self.p.type == 1:
            for i in range(4):
                self.zonePanel.sample_combos[i].connect("changed", self.on_zone_sample_changed, self.zonePanel.sample_combos[i], self.keygroupEditor.zone_labels[keygroup_index][i])
            
    def update_status(self, soattr, sovalue):
        self.statuslabel.set_text("Setting: " + soattr + " " + str(sovalue))
        
    def on_toggled_callback(self, widget, data):
        #print "%s was toggled %s" % (data, ("OFF", "ON")[widget.get_active()])
        if widget.get_active():
            self.change_keygroup(data-1) # index is +1

    def on_entryProgramName_changed(self, widget):
        if(self.p):
            self.p.set_name(widget.get_text())

            window = self.editor.parent
            if window and getattr(window, "p", None):
                window.update_title()




""" Python equivalent of akai section sampletools

Sample
"""

__author__ =  'Walco van Loon'
__version__ =  '0.2'

from aksy.devices.akai.sysex import Command

import aksy.devices.akai.sysex_types

class Sampletools:
    def __init__(self, z48):
        self.sampler = z48
        self.get_no_items_cmd = Command('_', '\x1C\x01', 'sampletools', 'get_no_items', (), None)
        self.get_handles_cmd = Command('_', '\x1C\x02\x00', 'sampletools', 'get_handles', (), None)
        self.get_names_cmd = Command('_', '\x1C\x02\x01', 'sampletools', 'get_names', (), None)
        self.get_handles_names_cmd = Command('_', '\x1C\x02\x02', 'sampletools', 'get_handles_names', (), None)
        self.get_handles_modified_cmd = Command('_', '\x1C\x02\x03', 'sampletools', 'get_handles_modified', (aksy.devices.akai.sysex_types.BYTE,), None)
        self.set_curr_by_handle_cmd = Command('_', '\x1C\x03', 'sampletools', 'set_curr_by_handle', (aksy.devices.akai.sysex_types.DWORD,), None)
        self.set_curr_by_name_cmd = Command('_', '\x1C\x04', 'sampletools', 'set_curr_by_name', (aksy.devices.akai.sysex_types.STRING,), None)
        self.get_curr_handle_cmd = Command('_', '\x1C\x05', 'sampletools', 'get_curr_handle', (), None)
        self.get_curr_name_cmd = Command('_', '\x1C\x06', 'sampletools', 'get_curr_name', (), None)
        self.get_name_by_handle_cmd = Command('_', '\x1C\x07', 'sampletools', 'get_name_by_handle', (aksy.devices.akai.sysex_types.DWORD,), None)
        self.get_handle_by_name_cmd = Command('_', '\x1C\x08', 'sampletools', 'get_handle_by_name', (aksy.devices.akai.sysex_types.STRING,), None)
        self.delete_all_cmd = Command('_', '\x1C\x09', 'sampletools', 'delete_all', (), None)
        self.delete_curr_cmd = Command('_', '\x1C\x0A', 'sampletools', 'delete_curr', (), None)
        self.delete_by_handle_cmd = Command('_', '\x1C\x0B', 'sampletools', 'delete_by_handle', (aksy.devices.akai.sysex_types.DWORD,), None)
        self.rename_curr_cmd = Command('_', '\x1C\x0C', 'sampletools', 'rename_curr', (aksy.devices.akai.sysex_types.STRING,), None)
        self.rename_by_handle_cmd = Command('_', '\x1C\x0D', 'sampletools', 'rename_by_handle', (aksy.devices.akai.sysex_types.DWORD, aksy.devices.akai.sysex_types.STRING), None)
        self.set_tag_bit_cmd = Command('_', '\x1C\x0E', 'sampletools', 'set_tag_bit', (aksy.devices.akai.sysex_types.BYTE, aksy.devices.akai.sysex_types.BYTE), None)
        self.get_tag_bitmap_cmd = Command('_', '\x1C\x0F', 'sampletools', 'get_tag_bitmap', (), None)
        self.get_curr_modified_cmd = Command('_', '\x1C\x10', 'sampletools', 'get_curr_modified', (), None)
        self.get_modified_cmd = Command('_', '\x1C\x11', 'sampletools', 'get_modified', (), None)
        self.delete_tagged_cmd = Command('_', '\x1C\x18', 'sampletools', 'delete_tagged', (aksy.devices.akai.sysex_types.BYTE,), None)
        self.play_cmd = Command('_', '\x1C\x40', 'sampletools', 'play', (aksy.devices.akai.sysex_types.BYTE, aksy.devices.akai.sysex_types.BOOL), None)
        self.stop_cmd = Command('_', '\x1C\x41', 'sampletools', 'stop', (), None)
        self.play_until_cmd = Command('_', '\x1C\x42', 'sampletools', 'play_until', (aksy.devices.akai.sysex_types.BYTE, aksy.devices.akai.sysex_types.QWORD), None)
        self.play_from_cmd = Command('_', '\x1C\x43', 'sampletools', 'play_from', (aksy.devices.akai.sysex_types.BYTE, aksy.devices.akai.sysex_types.QWORD), None)
        self.play_over_cmd = Command('_', '\x1C\x44', 'sampletools', 'play_over', (aksy.devices.akai.sysex_types.BYTE, aksy.devices.akai.sysex_types.QWORD), None)
        self.play_loop_cmd = Command('_', '\x1C\x45', 'sampletools', 'play_loop', (aksy.devices.akai.sysex_types.BYTE, aksy.devices.akai.sysex_types.BYTE), None)
        self.play_region_cmd = Command('_', '\x1C\x46', 'sampletools', 'play_region', (aksy.devices.akai.sysex_types.BYTE, aksy.devices.akai.sysex_types.BYTE), None)
        self.create_loop_cmd = Command('_', '\x1C\x48', 'sampletools', 'create_loop', (), None)
        self.delete_loop_cmd = Command('_', '\x1C\x49', 'sampletools', 'delete_loop', (aksy.devices.akai.sysex_types.BYTE,), None)
        self.create_region_cmd = Command('_', '\x1C\x4A', 'sampletools', 'create_region', (), None)
        self.delete_region_cmd = Command('_', '\x1C\x4B', 'sampletools', 'delete_region', (aksy.devices.akai.sysex_types.BYTE,), None)
        self.get_group_id_cmd = Command('_', '\x1F\x01', 'sampletools', 'get_group_id', (), None)
        self.get_trim_start_cmd = Command('_', '\x1F\x20', 'sampletools', 'get_trim_start', (), None)
        self.get_trim_end_cmd = Command('_', '\x1F\x21', 'sampletools', 'get_trim_end', (), None)
        self.get_trim_length_cmd = Command('_', '\x1F\x22', 'sampletools', 'get_trim_length', (), None)
        self.get_orig_pitch_cmd = Command('_', '\x1F\x24', 'sampletools', 'get_orig_pitch', (), None)
        self.get_cents_tune_cmd = Command('_', '\x1F\x25', 'sampletools', 'get_cents_tune', (), None)
        self.get_playback_mode_cmd = Command('_', '\x1F\x26', 'sampletools', 'get_playback_mode', (), None)
        self.get_loop_start_cmd = Command('_', '\x1F\x30', 'sampletools', 'get_loop_start', (aksy.devices.akai.sysex_types.BYTE,), None)
        self.get_loop_end_cmd = Command('_', '\x1F\x31', 'sampletools', 'get_loop_end', (aksy.devices.akai.sysex_types.BYTE,), None)
        self.get_loop_length_cmd = Command('_', '\x1F\x32', 'sampletools', 'get_loop_length', (aksy.devices.akai.sysex_types.BYTE,), None)
        self.get_loop_lock_cmd = Command('_', '\x1F\x33', 'sampletools', 'get_loop_lock', (aksy.devices.akai.sysex_types.BOOL,), None)
        self.get_loop_tune_cmd = Command('_', '\x1F\x34', 'sampletools', 'get_loop_tune', (aksy.devices.akai.sysex_types.BYTE,), None)
        self.get_loop_dir_cmd = Command('_', '\x1F\x35', 'sampletools', 'get_loop_dir', (aksy.devices.akai.sysex_types.BOOL,), None)
        self.get_loop_type_cmd = Command('_', '\x1F\x36', 'sampletools', 'get_loop_type', (aksy.devices.akai.sysex_types.BYTE,), None)
        self.get_no_loop_reps_cmd = Command('_', '\x1F\x37', 'sampletools', 'get_no_loop_reps', (aksy.devices.akai.sysex_types.BYTE,), None)
        self.get_no_loops_cmd = Command('_', '\x1F\x38', 'sampletools', 'get_no_loops', (), None)
        self.get_region_start_cmd = Command('_', '\x1F\x40', 'sampletools', 'get_region_start', (), None)
        self.get_region_end_cmd = Command('_', '\x1F\x41', 'sampletools', 'get_region_end', (aksy.devices.akai.sysex_types.BYTE,), None)
        self.get_region_length_cmd = Command('_', '\x1F\x42', 'sampletools', 'get_region_length', (), None)
        self.get_no_regions_cmd = Command('_', '\x1F\x44', 'sampletools', 'get_no_regions', (), None)
        self.get_sample_length_cmd = Command('_', '\x1F\x50', 'sampletools', 'get_sample_length', (), None)
        self.get_sample_rate_cmd = Command('_', '\x1F\x51', 'sampletools', 'get_sample_rate', (), None)
        self.get_bit_depth_cmd = Command('_', '\x1F\x52', 'sampletools', 'get_bit_depth', (), None)
        self.get_sample_type_cmd = Command('_', '\x1F\x54', 'sampletools', 'get_sample_type', (), None)
        self.get_no_channels_cmd = Command('_', '\x1F\x55', 'sampletools', 'get_no_channels', (), None)
        self.set_group_id_cmd = Command('_', '\x1E\x01', 'sampletools', 'set_group_id', (aksy.devices.akai.sysex_types.BYTE,), None)
        self.set_trim_start_cmd = Command('_', '\x1E\x20', 'sampletools', 'set_trim_start', (aksy.devices.akai.sysex_types.QWORD,), None)
        self.set_trim_end_cmd = Command('_', '\x1E\x21', 'sampletools', 'set_trim_end', (aksy.devices.akai.sysex_types.QWORD,), None)
        self.set_trim_length_cmd = Command('_', '\x1E\x22', 'sampletools', 'set_trim_length', (aksy.devices.akai.sysex_types.QWORD,), None)
        self.set_orig_pitch_cmd = Command('_', '\x1E\x24', 'sampletools', 'set_orig_pitch', (aksy.devices.akai.sysex_types.BYTE,), None)
        self.set_tune_cmd = Command('_', '\x1E\x25', 'sampletools', 'set_tune', (aksy.devices.akai.sysex_types.SWORD,), None)
        self.set_playback_mode_cmd = Command('_', '\x1E\x26', 'sampletools', 'set_playback_mode', (aksy.devices.akai.sysex_types.BYTE,), None)
        self.set_loop_start_cmd = Command('_', '\x1E\x30', 'sampletools', 'set_loop_start', (aksy.devices.akai.sysex_types.BYTE, aksy.devices.akai.sysex_types.QWORD), None)
        self.set_loop_end_cmd = Command('_', '\x1E\x31', 'sampletools', 'set_loop_end', (aksy.devices.akai.sysex_types.BYTE, aksy.devices.akai.sysex_types.QWORD), None)
        self.set_loop_length_cmd = Command('_', '\x1E\x32', 'sampletools', 'set_loop_length', (aksy.devices.akai.sysex_types.BYTE, aksy.devices.akai.sysex_types.QWORD), None)
        self.set_loop_lock_cmd = Command('_', '\x1E\x33', 'sampletools', 'set_loop_lock', (aksy.devices.akai.sysex_types.BYTE, aksy.devices.akai.sysex_types.BOOL), None)
        self.set_loop_tune_cmd = Command('_', '\x1E\x34', 'sampletools', 'set_loop_tune', (aksy.devices.akai.sysex_types.BYTE, aksy.devices.akai.sysex_types.SBYTE), None)
        self.set_loop_direction_cmd = Command('_', '\x1E\x35', 'sampletools', 'set_loop_direction', (aksy.devices.akai.sysex_types.BYTE, aksy.devices.akai.sysex_types.BYTE), None)
        self.set_loop_type_cmd = Command('_', '\x1E\x36', 'sampletools', 'set_loop_type', (aksy.devices.akai.sysex_types.BYTE, aksy.devices.akai.sysex_types.BYTE), None)
        self.set_no_loop_reps_cmd = Command('_', '\x1E\x37', 'sampletools', 'set_no_loop_reps', (aksy.devices.akai.sysex_types.BYTE,), None)
        self.set_region_start_cmd = Command('_', '\x1E\x40', 'sampletools', 'set_region_start', (aksy.devices.akai.sysex_types.BYTE, aksy.devices.akai.sysex_types.QWORD), None)
        self.set_region_end_cmd = Command('_', '\x1E\x41', 'sampletools', 'set_region_end', (aksy.devices.akai.sysex_types.BYTE, aksy.devices.akai.sysex_types.QWORD), None)
        self.set_region_length_cmd = Command('_', '\x1E\x42', 'sampletools', 'set_region_length', (aksy.devices.akai.sysex_types.BYTE, aksy.devices.akai.sysex_types.QWORD), None)

    def get_no_items(self):
        """Get number of items in memory

        Returns:
            DWORD
        """
        return self.sampler.execute(self.get_no_items_cmd, ())

    def get_handles(self):
        """Get Sample handles
        """
        return self.sampler.execute(self.get_handles_cmd, ())

    def get_names(self):
        """Get sample names

        Returns:
            STRINGARRAY
        """
        return self.sampler.execute(self.get_names_cmd, ())

    def get_handles_names(self):
        """Get list of sample handles and names

        Returns:
            HANDLENAMEARRAY
        """
        return self.sampler.execute(self.get_handles_names_cmd, ())

    def get_handles_modified(self, arg0):
        """Get a list of modified/tagged samples

        Returns:
            HANDLENAMEARRAY
        """
        return self.sampler.execute(self.get_handles_modified_cmd, (arg0, ))

    def set_curr_by_handle(self, arg0):
        """Select current item by handle
        """
        return self.sampler.execute(self.set_curr_by_handle_cmd, (arg0, ))

    def set_curr_by_name(self, arg0):
        """Select current item by name
        """
        return self.sampler.execute(self.set_curr_by_name_cmd, (arg0, ))

    def get_curr_handle(self):
        """Get handle of current item

        Returns:
            DWORD
        """
        return self.sampler.execute(self.get_curr_handle_cmd, ())

    def get_curr_name(self):
        """Get name of current item

        Returns:
            STRING
        """
        return self.sampler.execute(self.get_curr_name_cmd, ())

    def get_name_by_handle(self, arg0):
        """Get item name from handle

        Returns:
            STRING
        """
        return self.sampler.execute(self.get_name_by_handle_cmd, (arg0, ))

    def get_handle_by_name(self, arg0):
        """Get item handle from name

        Returns:
            DWORD
        """
        return self.sampler.execute(self.get_handle_by_name_cmd, (arg0, ))

    def delete_all(self):
        """Delete ALL items from memory
        """
        return self.sampler.execute(self.delete_all_cmd, ())

    def delete_curr(self):
        """Delete current item from memory
        """
        return self.sampler.execute(self.delete_curr_cmd, ())

    def delete_by_handle(self, arg0):
        """Delete item represented by handle <Data1>
        """
        return self.sampler.execute(self.delete_by_handle_cmd, (arg0, ))

    def rename_curr(self, arg0):
        """Rename current item
        """
        return self.sampler.execute(self.rename_curr_cmd, (arg0, ))

    def rename_by_handle(self, arg0, arg1):
        """Rename item represented by handle <Data1>
        """
        return self.sampler.execute(self.rename_by_handle_cmd, (arg0, arg1, ))

    def set_tag_bit(self, arg0, arg1):
        """Set Tag Bit <Data1> = bit to set, <Data2> = (0=OFF, 1=ON) BYTE(0, 1) <Data3> = (0=CURRENT, 1=ALL)
        """
        return self.sampler.execute(self.set_tag_bit_cmd, (arg0, arg1, ))

    def get_tag_bitmap(self):
        """Get Tag Bitmap
        """
        return self.sampler.execute(self.get_tag_bitmap_cmd, ())

    def get_curr_modified(self):
        """Get name of current item with modified/tagged info

        Returns:
            STRINGARRAY
        """
        return self.sampler.execute(self.get_curr_modified_cmd, ())

    def get_modified(self):
        """Get modified state of current item.

        Returns:
            BYTE
        """
        return self.sampler.execute(self.get_modified_cmd, ())

    def delete_tagged(self, arg0):
        """Delete tagged items <Data1> = tag bit
        """
        return self.sampler.execute(self.delete_tagged_cmd, (arg0, ))

    def play(self, arg0, arg1):
        """Start auditioning the current sample <Data1> = velocity <Data2> =(NO LOOPING, 1=LOOPING)
        """
        return self.sampler.execute(self.play_cmd, (arg0, arg1, ))

    def stop(self):
        """Stop playback of the current sample
        """
        return self.sampler.execute(self.stop_cmd, ())

    def play_until(self, arg0, arg1):
        """Play To <Data1> = velocity, <Data2> = sample position
        """
        return self.sampler.execute(self.play_until_cmd, (arg0, arg1, ))

    def play_from(self, arg0, arg1):
        """Play From <Data1> = velocity, <Data2> = sample position
        """
        return self.sampler.execute(self.play_from_cmd, (arg0, arg1, ))

    def play_over(self, arg0, arg1):
        """Play Over <Data1> = velocity, <Data2> = sample position
        """
        return self.sampler.execute(self.play_over_cmd, (arg0, arg1, ))

    def play_loop(self, arg0, arg1):
        """Play Loop <Data1> = velocity, <Data2> = loop index
        """
        return self.sampler.execute(self.play_loop_cmd, (arg0, arg1, ))

    def play_region(self, arg0, arg1):
        """Play Region <Data1> = velocity, <Data2> = region index
        """
        return self.sampler.execute(self.play_region_cmd, (arg0, arg1, ))

    def create_loop(self):
        """Create New Loop
        """
        return self.sampler.execute(self.create_loop_cmd, ())

    def delete_loop(self, arg0):
        """Delete Loop <Data1> = index
        """
        return self.sampler.execute(self.delete_loop_cmd, (arg0, ))

    def create_region(self):
        """Create Region
        """
        return self.sampler.execute(self.create_region_cmd, ())

    def delete_region(self, arg0):
        """Delete Region <Data1> = index
        """
        return self.sampler.execute(self.delete_region_cmd, (arg0, ))

    def get_group_id(self):
        """Get Group ID

        Returns:
            BYTE
        """
        return self.sampler.execute(self.get_group_id_cmd, ())

    def get_trim_start(self):
        """Get Trim Start

        Returns:
            QWORD
        """
        return self.sampler.execute(self.get_trim_start_cmd, ())

    def get_trim_end(self):
        """Get Trim End

        Returns:
            QWORD
        """
        return self.sampler.execute(self.get_trim_end_cmd, ())

    def get_trim_length(self):
        """Get Trim Length

        Returns:
            QWORD
        """
        return self.sampler.execute(self.get_trim_length_cmd, ())

    def get_orig_pitch(self):
        """Get Original Pitch

        Returns:
            BYTE
        """
        return self.sampler.execute(self.get_orig_pitch_cmd, ())

    def get_cents_tune(self):
        """Get Cents Tune (+-3600)

        Returns:
            SWORD
        """
        return self.sampler.execute(self.get_cents_tune_cmd, ())

    def get_playback_mode(self):
        """Get Playback Mode, where <Data1> = (0=NO LOOPING, 1=LOOPING, 2=ONE SHOT)

        Returns:
            BYTE
        """
        return self.sampler.execute(self.get_playback_mode_cmd, ())

    def get_loop_start(self, arg0):
        """Get Loop Start <Data1> = loop index

        Returns:
            QWORD
        """
        return self.sampler.execute(self.get_loop_start_cmd, (arg0, ))

    def get_loop_end(self, arg0):
        """Get Loop End <Data1> = loop index

        Returns:
            QWORD
        """
        return self.sampler.execute(self.get_loop_end_cmd, (arg0, ))

    def get_loop_length(self, arg0):
        """Get Loop Length <Data1> = loop index

        Returns:
            QWORD
        """
        return self.sampler.execute(self.get_loop_length_cmd, (arg0, ))

    def get_loop_lock(self, arg0):
        """Get Loop Lock <Data1> = (0=OFF, 1=ON)

        Returns:
            BYTE
        """
        return self.sampler.execute(self.get_loop_lock_cmd, (arg0, ))

    def get_loop_tune(self, arg0):
        """Get Loop Tune (+-50)

        Returns:
            SBYTE
        """
        return self.sampler.execute(self.get_loop_tune_cmd, (arg0, ))

    def get_loop_dir(self, arg0):
        """Get Loop Direction <Data1> = (0=FORWARDS, 1=ALTERNATING)

        Returns:
            BYTE
        """
        return self.sampler.execute(self.get_loop_dir_cmd, (arg0, ))

    def get_loop_type(self, arg0):
        """Get Loop Type <Data1> = (0=LOOP IN REL, 1=LOOP UNTIL REL)

        Returns:
            BYTE
        """
        return self.sampler.execute(self.get_loop_type_cmd, (arg0, ))

    def get_no_loop_reps(self, arg0):
        """Get Number of Loop Repetitions

        Returns:
            BYTE
        """
        return self.sampler.execute(self.get_no_loop_reps_cmd, (arg0, ))

    def get_no_loops(self):
        """Get Number of Loops

        Returns:
            BYTE
        """
        return self.sampler.execute(self.get_no_loops_cmd, ())

    def get_region_start(self):
        """Get Region Start <Data1> = Region Num (0-31), <Reply1> = start

        Returns:
            QWORD
        """
        return self.sampler.execute(self.get_region_start_cmd, ())

    def get_region_end(self, arg0):
        """Get Region End <Data1> = Region Num (0-31) <Reply1> = end

        Returns:
            QWORD
        """
        return self.sampler.execute(self.get_region_end_cmd, (arg0, ))

    def get_region_length(self):
        """Get Region Length <Data1> = Region Num (0-31) <Reply1> = length BYTE

        Returns:
            QWORD
        """
        return self.sampler.execute(self.get_region_length_cmd, ())

    def get_no_regions(self):
        """Get Number of Regions

        Returns:
            BYTE
        """
        return self.sampler.execute(self.get_no_regions_cmd, ())

    def get_sample_length(self):
        """Get Sample Length

        Returns:
            QWORD
        """
        return self.sampler.execute(self.get_sample_length_cmd, ())

    def get_sample_rate(self):
        """Get Sample Rate [Hz]

        Returns:
            DWORD
        """
        return self.sampler.execute(self.get_sample_rate_cmd, ())

    def get_bit_depth(self):
        """Get Sample Bit-Depth [bits]

        Returns:
            BYTE
        """
        return self.sampler.execute(self.get_bit_depth_cmd, ())

    def get_sample_type(self):
        """Get Sample Type <Reply> = (0=RAM, 1=VIRTUAL)

        Returns:
            BOOL
        """
        return self.sampler.execute(self.get_sample_type_cmd, ())

    def get_no_channels(self):
        """Get Number of Channels

        Returns:
            BYTE
        """
        return self.sampler.execute(self.get_no_channels_cmd, ())

    def set_group_id(self, arg0):
        """Set Group ID
        """
        return self.sampler.execute(self.set_group_id_cmd, (arg0, ))

    def set_trim_start(self, arg0):
        """Set Trim Start
        """
        return self.sampler.execute(self.set_trim_start_cmd, (arg0, ))

    def set_trim_end(self, arg0):
        """Set Trim End
        """
        return self.sampler.execute(self.set_trim_end_cmd, (arg0, ))

    def set_trim_length(self, arg0):
        """Set Trim Length
        """
        return self.sampler.execute(self.set_trim_length_cmd, (arg0, ))

    def set_orig_pitch(self, arg0):
        """Set Original Pitch
        """
        return self.sampler.execute(self.set_orig_pitch_cmd, (arg0, ))

    def set_tune(self, arg0):
        """Set Cents Tune
        """
        return self.sampler.execute(self.set_tune_cmd, (arg0, ))

    def set_playback_mode(self, arg0):
        """Set Playback Mode, where arg0= (0=NO LOOPING, 1=LOOPING, 2=ONE SHOT)
        """
        return self.sampler.execute(self.set_playback_mode_cmd, (arg0, ))

    def set_loop_start(self, arg0, arg1):
        """Set Loop Start <Data1> = loop index
        """
        return self.sampler.execute(self.set_loop_start_cmd, (arg0, arg1, ))

    def set_loop_end(self, arg0, arg1):
        """Set Loop End
        """
        return self.sampler.execute(self.set_loop_end_cmd, (arg0, arg1, ))

    def set_loop_length(self, arg0, arg1):
        """Set Loop Length
        """
        return self.sampler.execute(self.set_loop_length_cmd, (arg0, arg1, ))

    def set_loop_lock(self, arg0, arg1):
        """Set Loop Lock <Data1> = (0=OFF, 1=ON)
        """
        return self.sampler.execute(self.set_loop_lock_cmd, (arg0, arg1, ))

    def set_loop_tune(self, arg0, arg1):
        """Set Loop Tune (0-50)
        """
        return self.sampler.execute(self.set_loop_tune_cmd, (arg0, arg1, ))

    def set_loop_direction(self, arg0, arg1):
        """Set Loop Direction <Data1> = (0=FORWARDS, 1=ALTERNATING)
        """
        return self.sampler.execute(self.set_loop_direction_cmd, (arg0, arg1, ))

    def set_loop_type(self, arg0, arg1):
        """Set Loop Type <Data1> = (0=LOOP IN REL, 1=LOOP UNTIL REL)
        """
        return self.sampler.execute(self.set_loop_type_cmd, (arg0, arg1, ))

    def set_no_loop_reps(self, arg0):
        """Set Number of Loop Repetitions (0=INFINITE)
        """
        return self.sampler.execute(self.set_no_loop_reps_cmd, (arg0, ))

    def set_region_start(self, arg0, arg1):
        """Set Region Start <Data1> = Region Num, <Data2> = start
        """
        return self.sampler.execute(self.set_region_start_cmd, (arg0, arg1, ))

    def set_region_end(self, arg0, arg1):
        """Set Region End <Data1> = Region Num <Data2> = end
        """
        return self.sampler.execute(self.set_region_end_cmd, (arg0, arg1, ))

    def set_region_length(self, arg0, arg1):
        """Set Region Length <Data1> = Region Num <Data2> = length
        """
        return self.sampler.execute(self.set_region_length_cmd, (arg0, arg1, ))




import samplerobject

class Envelope(samplerobject.SamplerObject):
    def __init__(self, keygroup, index, xoffset=10, yoffset=10):
        samplerobject.SamplerObject.__init__(self, keygroup.s, keygroup, "keygrouptools", index)
        self.keygroup_index = keygroup.index
        self.need_index_for_set = True
        self.need_index_in_arguments = True
        self.attrs = ["envelope_rate1", "envelope_level1", "envelope_rate2", "envelope_level2",  "envelope_rate3", "envelope_level3", "envelope_rate4", "envelope_level4", "envelope_reference"]
        self.abbr = {
                    'envelope_rate1' : 'R1', 'envelope_level1' : 'L1',
                    'envelope_rate2' : 'R2', 'envelope_level2' : 'L2',
                    'envelope_rate3' : 'R3', 'envelope_level3' : 'L3',
                    'envelope_rate4' : 'R4', 'envelope_level4' : 'L4',
                    'envelope_reference' : 'REF'
                    }
        self.xoffset = xoffset
        self.yoffset = yoffset 
        self.set_envelope(keygroup, index)
        self.precache()
    
    def get_handle(self):
        return self.keygroup.get_handle()
    
    def updateNode(self, nodeIndex, rate, level):
        # stop it
        level = max(0,min(100,level))
        rate = max(0,min(100,rate))

        rateattr = "envelope_rate" + str(nodeIndex + 1) 
        levelattr = "envelope_level" + str(nodeIndex + 1)
        
        self.set(rateattr, rate) 
        self.set(levelattr, level) 

    def set_envelope(self, keygroup, index):
        if type(self.index) != int:
            raise Exception("Not an int.")
        self.keygroup = keygroup
        self.index = index 



import gtk

import panelbase, rangewidget
from aksui.ak import modulationmatrix

"""
    sources      = ['NONE','MODWHEEL','BEND UP','BEND DOWN',
                    'AFTERTOUCH','VELOCITY','BIPOLAR VELOCITY','OFF VELOCITY',
                    'KEYBOARD','LFO1','LFO2','AMP ENV',
                    'FILTER ENV','AUX ENV','MIDI CTRL','Q-LINK 1',
                    'Q-LINK 2','Q-LINK 3','Q-LINK 4','Q-LINK 5',
                    'Q-LINK 6','Q-LINK 7','Q-LINK 8','*MODWHEEL',
                    '*BEND UP','*BEND DOWN','*Q-LINK 1','*Q-LINK 2',
                    '*Q-LINK 3','*Q-LINK 4','*Q-LINK 5','*Q-LINK 6',
                    '*Q-LINK 7','*Q-LINK 8','*LFO1','*LFO2','*MIDI CTRL']

    # should be a total of 52 (0 based), qlink adds 4 'EXT' options
    destinations = ['NONE',           'AMPLITUDE',    'PAN',            'PITCH',
                    'LFO1 RATE',      'LFO1 DEPTH',   'LFO1 DELAY',     'LFO1 PHASE',
                    'LFO1 OFFSET',    'LFO2 RATE',    'LFO2 DEPTH',     'LFO2 DELAY',
                    'LFO2 PHASE',     'LFO2 OFFSET',  'CUTOFF',  'RESONANCE',
                    'TR 1 CUTOFF',    'TR 1 RES',     'TR 2 CUTOFF',    'TR 2 RES',
                    'TR 3 CUTOFF',    'TR 3 RES',     'AMP ENV ATTACK', 'AMP ENV DECAY',
                    'AMP ENV RELEASE','FILT ENV R1',  'FILT ENV R2',    'FILT ENV R4',
                    'AUX ENV R1',     'AUX ENV R2',   'AUX ENV R4',     'ZONE CROSSFADE',
                    'ZONE 1 LEVEL',   'ZONE 1 PAN',   'ZONE 1 PITCH',   'ZONE 1 START', 'ZONE 1 FILTER'
                    'ZONE 2 LEVEL',   'ZONE 2 PAN',   'ZONE 2 PITCH',   'ZONE 2 START', 'ZONE 2 FILTER'
                    'ZONE 3 LEVEL',   'ZONE 3 PAN',   'ZONE 3 PITCH',   'ZONE 3 START', 'ZONE 3 FILTER'
                    'ZONE 4 LEVEL',   'ZONE 4 PAN',   'ZONE 4 PITCH',   'ZONE 4 START', 'ZONE 4 FILTER']
"""

class ModMatrix(panelbase.PanelBase):
    def __init__(self, so, cb, sources = ['MODWHEEL', 'BIPOLAR VELOCITY', 'AMP ENV', 'FILTER ENV', 'AUX ENV', 'LFO1', 'LFO2', '*MODWHEEL', '*LFO1', '*LFO2'], destinations = ['AMPLITUDE', 'PAN', 'PITCH', 'CUTOFF', 'RESONANCE', 'ZONE CROSSFADE', 'AMP ENV ATTACK', 'AMP ENV DECAY', 'AMP ENV RELEASE', 'LFO1 RATE', 'LFO1 DEPTH', 'LFO1 DELAY', 'LFO1 PHASE', 'LFO1 OFFSET', 'LFO2 RATE', 'LFO2 DEPTH', 'LFO2 DELAY', 'LFO2 PHASE','LFO2 OFFSET']):
        # doing this by mod source/dest name because it's easier to read, probably would be slightly faster if it used index # 
        assert sources != None
        assert destinations != None

        self.sources = sources
        self.destinations = destinations

        panelbase.PanelBase.__init__(self, so, "Modulation Matrix", cb)

        self.setup(so)

    def setup(self, so):
        self.clear_children(True)

        self.s = so.s
        self.so = so

        if len(so.mod_matrix) == 0:
            so.load_matrix()

        self.table = gtk.Table(len(self.sources) + 1, len(self.destinations) + 1, False)
        t = self.table

        for source in self.sources:
            i = self.sources.index(source)
            source_label = gtk.Label("<span size='smaller'>%s</span>" % source)
            source_label.set_property("angle", 90)
            source_label.set_use_markup(True)
            t.attach(source_label, i+1, i+2, 0, 1)

        for dest in self.destinations:
            i = self.destinations.index(dest)
            dest_index = modulationmatrix.ModulationMatrix.destinations.index(dest)

            dest_label = gtk.Label("<span size='smaller'>%s</span>" % dest)
            dest_label.set_use_markup(True)
            t.attach(dest_label, 0, 1, i + 1, i + 2)

            for source in self.sources:
                j = self.sources.index(source)
                source_index = modulationmatrix.ModulationMatrix.sources.index(source)
                mod_name = "MOD_%d_%d" % (source_index, dest_index)
                mod_knob = rangewidget.AkKnobWidget(self.so, mod_name, -100, 100, 1, units="", mod_destination=dest) # filter env to res
                t.attach(mod_knob, j + 1, j + 2, i + 1, i + 2, False, False) 

        self.pack_start(t, False, False, 1)
        self.show_all()



from aksui.UI import base
	
 	
class ProgramDetails(base.Base):
	    def __init__(self, program):
 	        base.Base.__init__(self, program, "windowProgramDetails")
 	        self.editor.connect("delete-event", self.on_delete_event)
 	        self.set_samplerobject(program)
 	       
 	    def on_delete_event(self, widget, event):
 	        # nooooo
 	        self.editor.hide_all()
 	        return True




""" Python equivalent of akai section programtools

Methods to manipulate in-memory programs
"""

__author__ =  'Walco van Loon'
__version__ =  '0.2'

from aksy.devices.akai.sysex import Command

import aksy.devices.akai.sysex_types

class Programtools:
    def __init__(self, s56k):
        self.sampler = s56k
        self.create_empty_cmd = Command('^', '\x0a\x02', 'programtools', 'create_empty', (aksy.devices.akai.sysex_types.STRING,), (), aksy.devices.akai.sysex_types.S56K_USERREF)
        self.create_new_cmd = Command('^', '\x0a\x03', 'programtools', 'create_new', (aksy.devices.akai.sysex_types.BYTE, aksy.devices.akai.sysex_types.STRING), (), aksy.devices.akai.sysex_types.S56K_USERREF)
        self.set_curr_by_name_cmd = Command('^', '\x0a\x05', 'programtools', 'set_curr_by_name', (aksy.devices.akai.sysex_types.STRING,), (), aksy.devices.akai.sysex_types.S56K_USERREF)
        self.set_curr_by_handle_cmd = Command('^', '\x0a\x06', 'programtools', 'set_curr_by_handle', (aksy.devices.akai.sysex_types.WORD,), (), aksy.devices.akai.sysex_types.S56K_USERREF)
        self.delete_all_cmd = Command('^', '\x0a\x07', 'programtools', 'delete_all', (), (), aksy.devices.akai.sysex_types.S56K_USERREF)
        self.delete_curr_cmd = Command('^', '\x0a\x08', 'programtools', 'delete_curr', (), (), aksy.devices.akai.sysex_types.S56K_USERREF)
        self.rename_curr_cmd = Command('^', '\x0a\x09', 'programtools', 'rename_curr', (aksy.devices.akai.sysex_types.STRING,), (), aksy.devices.akai.sysex_types.S56K_USERREF)
        self.set_program_no_cmd = Command('^', '\x0a\x0A', 'programtools', 'set_program_no', (aksy.devices.akai.sysex_types.BOOL,aksy.devices.akai.sysex_types.BYTE), (), aksy.devices.akai.sysex_types.S56K_USERREF)
        self.add_keygroups_cmd = Command('^', '\x0a\x0B', 'programtools', 'add_keygroups', (aksy.devices.akai.sysex_types.BYTE,), (), aksy.devices.akai.sysex_types.S56K_USERREF)
        self.delete_keygroup_cmd = Command('^', '\x0a\x0C', 'programtools', 'delete_keygroup', (aksy.devices.akai.sysex_types.BYTE,), (), aksy.devices.akai.sysex_types.S56K_USERREF)
        self.set_keygroup_xfade_cmd = Command('^', '\x0a\x0D', 'programtools', 'set_keygroup_xfade', (aksy.devices.akai.sysex_types.BOOL,), (), aksy.devices.akai.sysex_types.S56K_USERREF)
        self.get_no_items_cmd = Command('^', '\x0a\x10', 'programtools', 'get_no_items', (), (aksy.devices.akai.sysex_types.CWORD,), aksy.devices.akai.sysex_types.S56K_USERREF)
        self.get_program_no_cmd = Command('^', '\x0a\x11', 'programtools', 'get_program_no', (), (aksy.devices.akai.sysex_types.BOOL,aksy.devices.akai.sysex_types.BYTE), aksy.devices.akai.sysex_types.S56K_USERREF)
        self.get_index_cmd = Command('^', '\x0a\x12', 'programtools', 'get_index', (), (aksy.devices.akai.sysex_types.CWORD,), aksy.devices.akai.sysex_types.S56K_USERREF)
        self.get_name_cmd = Command('^', '\x0a\x13', 'programtools', 'get_name', (), (aksy.devices.akai.sysex_types.STRINGARRAY,), aksy.devices.akai.sysex_types.S56K_USERREF)
        self.get_no_keygroups_cmd = Command('^', '\x0a\x14', 'programtools', 'get_no_keygroups', (), (aksy.devices.akai.sysex_types.BYTE), aksy.devices.akai.sysex_types.S56K_USERREF)
        self.get_keygroup_xfade_cmd = Command('^', '\x0a\x15', 'programtools', 'get_keygroup_xfade', (), (aksy.devices.akai.sysex_types.BOOL), aksy.devices.akai.sysex_types.S56K_USERREF)
        self.get_program_numbers_cmd = Command('^', '\x0a\x18', 'programtools', 'get_program_numbers', (), (aksy.devices.akai.sysex_types.BOOL,aksy.devices.akai.sysex_types.BYTE), aksy.devices.akai.sysex_types.S56K_USERREF)
        self.get_names_cmd = Command('^', '\x0a\x19', 'programtools', 'get_names', (), (aksy.devices.akai.sysex_types.STRINGARRAY,), aksy.devices.akai.sysex_types.S56K_USERREF)
        
        self.set_loudness_cmd = Command('^', '\x0a\x20', 'programtools', 'set_loudness', (aksy.devices.akai.sysex_types.BYTE,), (), aksy.devices.akai.sysex_types.S56K_USERREF)
        self.set_velocity_sens_cmd = Command('^', '\x0a\x21', 'programtools', 'set_velocity_sens', (aksy.devices.akai.sysex_types.SBYTE,), (), aksy.devices.akai.sysex_types.S56K_USERREF)
        self.set_amp_modulation_source_cmd = Command('^', '\x0a\x22', 'programtools', 'set_amp_modulation_source', (aksy.devices.akai.sysex_types.BYTE,), (), aksy.devices.akai.sysex_types.S56K_USERREF)
        self.set_amp_modulation_value_cmd = Command('^', '\x0a\x23', 'programtools', 'set_amp_modulation_value', (aksy.devices.akai.sysex_types.BYTE, aksy.devices.akai.sysex_types.SBYTE), (), aksy.devices.akai.sysex_types.S56K_USERREF)
        self.set_pan_modulation_source_cmd = Command('^', '\x0a\x24', 'programtools', 'set_pan_modulation_source', (aksy.devices.akai.sysex_types.BYTE, aksy.devices.akai.sysex_types.SBYTE), (), aksy.devices.akai.sysex_types.S56K_USERREF)
        self.set_pan_modulation_value_cmd = Command('^', '\x0a\x25', 'programtools', 'set_pan_modulation_value', (), (), aksy.devices.akai.sysex_types.S56K_USERREF)
        self.get_loudness_cmd = Command('^', '\x0a\x28', 'programtools', 'get_loudness', (), (), aksy.devices.akai.sysex_types.S56K_USERREF)
        self.get_velocity_sens_cmd = Command('^', '\x0a\x29', 'programtools', 'get_velocity_sens', (), (), aksy.devices.akai.sysex_types.S56K_USERREF)
        self.get_amp_modulation_source_cmd = Command('^', '\x0a\x2A', 'programtools', 'get_amp_modulation_source', (), (), aksy.devices.akai.sysex_types.S56K_USERREF)
        self.get_amp_modulation_value_cmd = Command('^', '\x0a\x2B', 'programtools', 'get_amp_modulation_value', (), (), aksy.devices.akai.sysex_types.S56K_USERREF)
        self.get_pan_modulation_source_cmd = Command('^', '\x0a\x2C', 'programtools', 'get_pan_modulation_source', (), (), aksy.devices.akai.sysex_types.S56K_USERREF)
        self.get_pan_modulation_value_cmd = Command('^', '\x0a\x2D', 'programtools', 'get_pan_modulation_value', (), (), aksy.devices.akai.sysex_types.S56K_USERREF)
        self.set_semitone_tune_cmd = Command('^', '\x0a\x30', 'programtools', 'set_semitone_tune', (aksy.devices.akai.sysex_types.SBYTE,), (), aksy.devices.akai.sysex_types.S56K_USERREF)
        self.set_fine_tune_cmd = Command('^', '\x0a\x31', 'programtools', 'set_fine_tune', (aksy.devices.akai.sysex_types.SBYTE,), (), aksy.devices.akai.sysex_types.S56K_USERREF)
        self.set_tune_template_cmd = Command('^', '\x0a\x32', 'programtools', 'set_tune_template', (aksy.devices.akai.sysex_types.BYTE,), (), aksy.devices.akai.sysex_types.S56K_USERREF)
        self.set_user_tune_template_cmd = Command('^', '\x0a\x33', 'programtools', 'set_user_tune_template', (aksy.devices.akai.sysex_types.BYTE,), (), aksy.devices.akai.sysex_types.S56K_USERREF)
        self.set_key_cmd = Command('^', '\x0a\x34', 'programtools', 'set_key', (aksy.devices.akai.sysex_types.BYTE,), (), aksy.devices.akai.sysex_types.S56K_USERREF)
        self.get_semitone_tune_cmd = Command('^', '\x0a\x38', 'programtools', 'get_semitone_tune', (), (), aksy.devices.akai.sysex_types.S56K_USERREF)
        self.get_fine_tune_cmd = Command('^', '\x0a\x39', 'programtools', 'get_fine_tune', (), (), aksy.devices.akai.sysex_types.S56K_USERREF)
        self.get_tune_template_cmd = Command('^', '\x0a\x3A', 'programtools', 'get_tune_template', (), (), aksy.devices.akai.sysex_types.S56K_USERREF)
        self.get_user_tune_template_cmd = Command('^', '\x0a\x3B', 'programtools', 'get_user_tune_template', (), (), aksy.devices.akai.sysex_types.S56K_USERREF)
        self.get_key_cmd = Command('^', '\x0a\x3C', 'programtools', 'get_key', (), (), aksy.devices.akai.sysex_types.S56K_USERREF)
        self.set_pitch_bend_up_cmd = Command('^', '\x0a\x40', 'programtools', 'set_pitch_bend_up', (aksy.devices.akai.sysex_types.BYTE,), (), aksy.devices.akai.sysex_types.S56K_USERREF)
        self.set_pitch_bend_down_cmd = Command('^', '\x0a\x41', 'programtools', 'set_pitch_bend_down', (aksy.devices.akai.sysex_types.BYTE,), (), aksy.devices.akai.sysex_types.S56K_USERREF)
        self.set_bend_mode_cmd = Command('^', '\x0a\x42', 'programtools', 'set_bend_mode', (aksy.devices.akai.sysex_types.BYTE,), (), aksy.devices.akai.sysex_types.S56K_USERREF)
        self.set_aftertouch_value_cmd = Command('^', '\x0a\x43', 'programtools', 'set_aftertouch_value', (aksy.devices.akai.sysex_types.SBYTE,), (), aksy.devices.akai.sysex_types.S56K_USERREF)
        self.set_legato_setting_cmd = Command('^', '\x0a\x44', 'programtools', 'set_legato_setting', (aksy.devices.akai.sysex_types.BYTE,), (), aksy.devices.akai.sysex_types.S56K_USERREF)
        self.set_portamento_enabled_cmd = Command('^', '\x0a\x45', 'programtools', 'set_portamento_enabled', (aksy.devices.akai.sysex_types.BOOL,), (), aksy.devices.akai.sysex_types.S56K_USERREF)
        self.set_portamento_mode_cmd = Command('^', '\x0a\x46', 'programtools', 'set_portamento_mode', (aksy.devices.akai.sysex_types.BYTE,), (), aksy.devices.akai.sysex_types.S56K_USERREF)
        self.set_portamento_time_cmd = Command('^', '\x0a\x47', 'programtools', 'set_portamento_time', (aksy.devices.akai.sysex_types.BYTE,), (), aksy.devices.akai.sysex_types.S56K_USERREF)
        self.get_pitch_bend_up_cmd = Command('^', '\x0a\x48', 'programtools', 'get_pitch_bend_up', (), (), aksy.devices.akai.sysex_types.S56K_USERREF)
        self.get_pitch_bend_down_cmd = Command('^', '\x0a\x49', 'programtools', 'get_pitch_bend_down', (), (), aksy.devices.akai.sysex_types.S56K_USERREF)
        self.get_bend_mode_cmd = Command('^', '\x0a\x4A', 'programtools', 'get_bend_mode', (), (), aksy.devices.akai.sysex_types.S56K_USERREF)
        self.get_aftertouch_value_cmd = Command('^', '\x0a\x4B', 'programtools', 'get_aftertouch_value', (), (), aksy.devices.akai.sysex_types.S56K_USERREF)
        self.get_legato_setting_cmd = Command('^', '\x0a\x4C', 'programtools', 'get_legato_setting', (), (), aksy.devices.akai.sysex_types.S56K_USERREF)
        self.get_portamento_enabled_cmd = Command('^', '\x0a\x4D', 'programtools', 'get_portamento_enabled', (), (), aksy.devices.akai.sysex_types.S56K_USERREF)
        self.get_portamento_mode_cmd = Command('^', '\x0a\x4E', 'programtools', 'get_portamento_mode', (), (), aksy.devices.akai.sysex_types.S56K_USERREF)
        self.get_portamento_time_cmd = Command('^', '\x0a\x4F', 'programtools', 'get_portamento_time', (), (), aksy.devices.akai.sysex_types.S56K_USERREF)
        self.set_lfo_rate_cmd = Command('^', '\x0a\x50', 'programtools', 'set_lfo_rate', (aksy.devices.akai.sysex_types.BYTE, aksy.devices.akai.sysex_types.BYTE), (), aksy.devices.akai.sysex_types.S56K_USERREF)
        self.set_lfo_delay_cmd = Command('^', '\x0a\x51', 'programtools', 'set_lfo_delay', (aksy.devices.akai.sysex_types.BYTE, aksy.devices.akai.sysex_types.BYTE), (), aksy.devices.akai.sysex_types.S56K_USERREF)
        self.set_lfo_depth_cmd = Command('^', '\x0a\x52', 'programtools', 'set_lfo_depth', (aksy.devices.akai.sysex_types.BYTE, aksy.devices.akai.sysex_types.BYTE), (), aksy.devices.akai.sysex_types.S56K_USERREF)
        self.set_lfo_waveform_cmd = Command('^', '\x0a\x53', 'programtools', 'set_lfo_waveform', (aksy.devices.akai.sysex_types.BYTE, aksy.devices.akai.sysex_types.BYTE), (), aksy.devices.akai.sysex_types.S56K_USERREF)
        self.set_lfo_sync_cmd = Command('^', '\x0a\x54', 'programtools', 'set_lfo_sync', (aksy.devices.akai.sysex_types.BYTE, aksy.devices.akai.sysex_types.BOOL), (), aksy.devices.akai.sysex_types.S56K_USERREF)
        self.set_lfo_retrigger_cmd = Command('^', '\x0a\x55', 'programtools', 'set_lfo_retrigger', (aksy.devices.akai.sysex_types.BYTE, aksy.devices.akai.sysex_types.BOOL), (), aksy.devices.akai.sysex_types.S56K_USERREF)
        self.set_rate_modulation_source_cmd = Command('^', '\x0a\x56', 'programtools', 'set_rate_modulation_source', (aksy.devices.akai.sysex_types.BYTE, aksy.devices.akai.sysex_types.BOOL), (), aksy.devices.akai.sysex_types.S56K_USERREF)
        self.get_lfo_sync_cmd = Command('^', '\x0a\x64', 'programtools', 'get_lfo_sync', (aksy.devices.akai.sysex_types.BYTE,), (), aksy.devices.akai.sysex_types.S56K_USERREF)
        self.get_lfo_retrigger_cmd = Command('^', '\x0a\x65', 'programtools', 'get_lfo_retrigger', (aksy.devices.akai.sysex_types.BYTE,), (), aksy.devices.akai.sysex_types.S56K_USERREF)
        self.get_rate_modulation_source_cmd = Command('^', '\x0a\x66', 'programtools', 'get_rate_modulation_source', (aksy.devices.akai.sysex_types.BYTE,), (), aksy.devices.akai.sysex_types.S56K_USERREF)
        self.get_rate_modulation_value_cmd = Command('^', '\x0a\x67', 'programtools', 'get_rate_modulation_value', (aksy.devices.akai.sysex_types.BYTE,), (), aksy.devices.akai.sysex_types.S56K_USERREF)
        self.get_delay_modulation_source_cmd = Command('^', '\x0a\x68', 'programtools', 'get_delay_modulation_source', (aksy.devices.akai.sysex_types.BYTE,), (), aksy.devices.akai.sysex_types.S56K_USERREF)
        self.get_delay_modulation_value_cmd = Command('^', '\x0a\x69', 'programtools', 'get_delay_modulation_value', (aksy.devices.akai.sysex_types.BYTE,), (), aksy.devices.akai.sysex_types.S56K_USERREF)
        self.get_depth_modulation_source_cmd = Command('^', '\x0a\x6A', 'programtools', 'get_depth_modulation_source', (aksy.devices.akai.sysex_types.BYTE,), (), aksy.devices.akai.sysex_types.S56K_USERREF)
        self.get_depth_modulation_value_cmd = Command('^', '\x0a\x6B', 'programtools', 'get_depth_modulation_value', (aksy.devices.akai.sysex_types.BYTE,), (), aksy.devices.akai.sysex_types.S56K_USERREF)
        self.get_modwheel_cmd = Command('^', '\x0a\x6C', 'programtools', 'get_modwheel', (aksy.devices.akai.sysex_types.BYTE,), (), aksy.devices.akai.sysex_types.S56K_USERREF)
        self.get_aftertouch_cmd = Command('^', '\x0a\x6D', 'programtools', 'get_aftertouch', (aksy.devices.akai.sysex_types.BYTE,), (), aksy.devices.akai.sysex_types.S56K_USERREF)
        self.get_midi_clock_sync_enabled_cmd = Command('^', '\x0a\x6E', 'programtools', 'get_midi_clock_sync_enabled', (), (), aksy.devices.akai.sysex_types.S56K_USERREF)
        self.get_midi_clock_sync_division_cmd = Command('^', '\x0a\x6F', 'programtools', 'get_midi_clock_sync_division', (aksy.devices.akai.sysex_types.BYTE,), (), aksy.devices.akai.sysex_types.S56K_USERREF)
        self.set_pitch_modulation_source_cmd = Command('^', '\x0a\x70', 'programtools', 'set_pitch_modulation_source', (aksy.devices.akai.sysex_types.BYTE, aksy.devices.akai.sysex_types.BYTE), (), aksy.devices.akai.sysex_types.S56K_USERREF)
        self.set_filter_modulation_source_cmd = Command('^', '\x0a\x72', 'programtools', 'set_filter_modulation_source', (aksy.devices.akai.sysex_types.BYTE, aksy.devices.akai.sysex_types.BYTE), (), aksy.devices.akai.sysex_types.S56K_USERREF)
        self.get_pitch_modulation_source_cmd = Command('^', '\x0a\x74', 'programtools', 'get_pitch_modulation_source', (aksy.devices.akai.sysex_types.BYTE,), (), aksy.devices.akai.sysex_types.S56K_USERREF)
        self.get_filter_modulation_source_cmd = Command('^', '\x0a\x76', 'programtools', 'get_filter_modulation_source', (aksy.devices.akai.sysex_types.BYTE,), (), aksy.devices.akai.sysex_types.S56K_USERREF)

    def create_empty(self, arg0):
        """Create Program: <Data1> = Name
        """
        return self.sampler.execute(self.create_empty_cmd, (arg0, ))

    def create_new(self, arg0, arg1):
        """Create Program with keygroups. <Data1> = Name
        """
        return self.sampler.execute(self.create_new_cmd, (arg0, arg1, ))

    def set_curr_by_name(self, arg0):
        """Select Program (by name) to be current: <Data1> = Name
        """
        return self.sampler.execute(self.set_curr_by_name_cmd, (arg0, ))

    def set_curr_by_handle(self, arg0):
        """Select Program (by index) to be current: <Data1> = handle.
        """
        return self.sampler.execute(self.set_curr_by_handle_cmd, (arg0, ))

    def delete_all(self):
        """Delete ALL programs from memory
        """
        return self.sampler.execute(self.delete_all_cmd, ())

    def delete_curr(self):
        """Delete the currently selected Program from memory
        """
        return self.sampler.execute(self.delete_curr_cmd, ())

    def rename_curr(self, arg0):
        """Rename currently selected Program: <Data1> = Name
        """
        return self.sampler.execute(self.rename_curr_cmd, (arg0, ))

    def set_program_no(self, arg0, arg1=0):  # arg1 is optional if arg=0?
        """Set Program Number. <Data1>=1).
        """
        return self.sampler.execute(self.set_program_no_cmd, (arg0, arg1))

    def add_keygroups(self, arg0):
        """Add Keygroups to Program <Data1> = Number of Keygroups to add.
        """
        return self.sampler.execute(self.add_keygroups_cmd, (arg0, ))

    def delete_keygroup(self, arg0):
        """Delete Keygroup from Program: <Data1> = Number of the keygroup to delete. (zero-based)
        """
        return self.sampler.execute(self.delete_keygroup_cmd, (arg0, ))

    def set_keygroup_xfade(self, arg0, arg1):
        """Set Keygroup Crossfade <Data1>: 0=OFF; 1=ON
        """
        return self.sampler.execute(self.set_keygroup_xfade_cmd, (arg0, arg1, ))

    def get_no_items(self):
        """Get Number of Programs in memory

        Returns:
            WORD
        """
        return self.sampler.execute(self.get_no_items_cmd, ())

    def get_program_no(self):
        """Get Current Programs Program Number

        Returns:
            BOOL
            BYTE
        """
        return self.sampler.execute(self.get_program_no_cmd, ())

    def get_index(self):
        """Get Current Program Index (i.e., its position in memory)

        Returns:
            WORD
        """
        return self.sampler.execute(self.get_index_cmd, ())

    def get_name(self):
        """Get Current Program Name

        Returns:
            STRING
        """
        return self.sampler.execute(self.get_name_cmd, ())

    def get_no_keygroups(self):
        """Get Number of Keygroups in Current Program

        Returns:
            BYTE
        """
        return self.sampler.execute(self.get_no_keygroups_cmd, ())

    def get_keygroup_xfade(self):
        """Get Keygroup Crossfade

        Returns:
            BOOL
        """
        return self.sampler.execute(self.get_keygroup_xfade_cmd, ())

    def get_program_numbers(self):
        """Get the Program Numbers of all the Programs in memory

        Returns:
            BYTE
        """
        return self.sampler.execute(self.get_program_numbers_cmd, ())

    def get_names(self):
        """Get the names of all of the Programs in memory

        Returns:
            STRINGARRAY
        """
        return self.sampler.execute(self.get_names_cmd, ())

    def set_loudness(self, arg0):
        """Set Loudness. <Data1> = loudness value.
        """
        return self.sampler.execute(self.set_loudness_cmd, (arg0, ))

    def set_velocity_sens(self, arg0):
        """Set Velocity Sensitivity. Values range from 100 to +100, <Data1> = absolute value.
        """
        return self.sampler.execute(self.set_velocity_sens_cmd, (arg0, ))

    def set_amp_modulation_source(self, arg0):
        """Set Amp Mod Source. <Data1> = Modulation Source. (see Table 15)
        """
        return self.sampler.execute(self.set_amp_modulation_source_cmd, (arg0, ))

    def set_amp_modulation_value(self, arg0, arg1):
        """Set Amp Mod Value. <Data1> = Amp Mod (1 or 2)
        """
        return self.sampler.execute(self.set_amp_modulation_value_cmd, (arg0, arg1, ))

    def set_pan_modulation_source(self, arg0, arg1):
        """Set Pan Mod Source. <Data1> = Modulation Source. (see Table 15)
        """
        return self.sampler.execute(self.set_pan_modulation_source_cmd, (arg0, arg1, ))

    def set_pan_modulation_value(self):
        """Set Pan Mod Value. <Data1>0-100
        """
        return self.sampler.execute(self.set_pan_modulation_value_cmd, ())

    def get_loudness(self):
        """Get Loudness.

        Returns:
            BYTE
        """
        return self.sampler.execute(self.get_loudness_cmd, ())

    def get_velocity_sens(self):
        """Get Velocity Sensitivity.

        Returns:
            SBYTE
        """
        return self.sampler.execute(self.get_velocity_sens_cmd, ())

    def get_amp_modulation_source(self):
        """Get Amp Mod Source. <Data1> = Amp Mod (1 or 2)

        Returns:
            BYTE
        """
        return self.sampler.execute(self.get_amp_modulation_source_cmd, ())

    def get_amp_modulation_value(self):
        """Get Amp Mod Value. <Data1> = Amp Mod (1 or 2)

        Returns:
            BYTE
        """
        return self.sampler.execute(self.get_amp_modulation_value_cmd, ())

    def get_pan_modulation_source(self):
        """Get Pan Mod Source. <Data1> = Pan Mod (1, 2 or 3)

        Returns:
            BYTE
        """
        return self.sampler.execute(self.get_pan_modulation_source_cmd, ())

    def get_pan_modulation_value(self):
        """Get Pan Mod Value. <Data1> = Pan Mod (1, 2 or 3)

        Returns:
            BYTE
        """
        return self.sampler.execute(self.get_pan_modulation_value_cmd, ())

    def set_semitone_tune(self, arg0):
        """Semitone Tune
        """
        return self.sampler.execute(self.set_semitone_tune_cmd, (arg0, ))

    def set_fine_tune(self, arg0):
        """Fine Tune.
        """
        return self.sampler.execute(self.set_fine_tune_cmd, (arg0, ))

    def set_tune_template(self, arg0):
        """Tune Template, where <Data1> = template. 0=USER, 1=EVEN-TEMPERED, 2=ORCHESTRAL, 3=WERKMEISTER, 4=1/5 MEANTONE, 5=1/4 MEANTONE, 6=JUST, 7=ARABIAN.
        """
        return self.sampler.execute(self.set_tune_template_cmd, (arg0, ))

    def set_user_tune_template(self, arg0):
        """Set User Tune Template. All the values are sent one after the other starting at C. The format of each value is the same as for Item &31{49}. (i.e., 24 data bytes are representing all 12 notes.)
        """
        return self.sampler.execute(self.set_user_tune_template_cmd, (arg0, ))

    def set_key(self, arg0):
        """Set Key = <Data1> where: 0=C, 1=C#, 2=D, 3=Eb, 4=E, 5=F, 6=F#, 7=G, 8=G#, 9=A, 10=Bb, 11=B
        """
        return self.sampler.execute(self.set_key_cmd, (arg0, ))

    def get_semitone_tune(self):
        """Get Semitone Tune.

        Returns:
            SBYTE
        """
        return self.sampler.execute(self.get_semitone_tune_cmd, ())

    def get_fine_tune(self):
        """Get Fine Tune.

        Returns:
            SBYTE
        """
        return self.sampler.execute(self.get_fine_tune_cmd, ())

    def get_tune_template(self):
        """Get Tune Template.

        Returns:
            BYTE
        """
        return self.sampler.execute(self.get_tune_template_cmd, ())

    def get_user_tune_template(self):
        """Get User Tune Template.

        Returns:
            BYTE
        """
        return self.sampler.execute(self.get_user_tune_template_cmd, ())

    def get_key(self):
        """Get Key.

        Returns:
            BYTE
        """
        return self.sampler.execute(self.get_key_cmd, ())

    def set_pitch_bend_up(self, arg0):
        """Set Pitch Bend Up. <Data1> = semitones
        """
        return self.sampler.execute(self.set_pitch_bend_up_cmd, (arg0, ))

    def set_pitch_bend_down(self, arg0):
        """Set Pitch Bend Down. <Data1> = semitones
        """
        return self.sampler.execute(self.set_pitch_bend_down_cmd, (arg0, ))

    def set_bend_mode(self, arg0):
        """Set Bend Mode. <Data1> = mode, where 0=NORMAL, 1=HELD
        """
        return self.sampler.execute(self.set_bend_mode_cmd, (arg0, ))

    def set_aftertouch_value(self, arg0):
        """Set Aftertouch Value.
        """
        return self.sampler.execute(self.set_aftertouch_value_cmd, (arg0, ))

    def set_legato_setting(self, arg0):
        """Set Legato Setting <Data1> = mode, where 0=OFF, 1=ON
        """
        return self.sampler.execute(self.set_legato_setting_cmd, (arg0, ))

    def set_portamento_enabled(self, arg0):
        """Set Portamento Enable <Data1> = mode, where 0=OFF, 1=ON
        """
        return self.sampler.execute(self.set_portamento_enabled_cmd, (arg0, ))

    def set_portamento_mode(self, arg0):
        """Set Portamento Mode <Data1> = mode, where 0=TIME, 1=RATE
        """
        return self.sampler.execute(self.set_portamento_mode_cmd, (arg0, ))

    def set_portamento_time(self, arg0):
        """Set Portamento Time
        """
        return self.sampler.execute(self.set_portamento_time_cmd, (arg0, ))

    def get_pitch_bend_up(self):
        """Get Pitch Bend Up

        Returns:
            BYTE
        """
        return self.sampler.execute(self.get_pitch_bend_up_cmd, ())

    def get_pitch_bend_down(self):
        """Get Pitch Bend Down

        Returns:
            BYTE
        """
        return self.sampler.execute(self.get_pitch_bend_down_cmd, ())

    def get_bend_mode(self):
        """Get Bend Mode

        Returns:
            BYTE
        """
        return self.sampler.execute(self.get_bend_mode_cmd, ())

    def get_aftertouch_value(self):
        """Get Aftertouch Value

        Returns:
            SBYTE
        """
        return self.sampler.execute(self.get_aftertouch_value_cmd, ())

    def get_legato_setting(self):
        """Get Legato Setting

        Returns:
            BYTE
        """
        return self.sampler.execute(self.get_legato_setting_cmd, ())

    def get_portamento_enabled(self):
        """Get Portamento Enable

        Returns:
            BOOL
        """
        return self.sampler.execute(self.get_portamento_enabled_cmd, ())

    def get_portamento_mode(self):
        """Get Portamento Mode

        Returns:
            BYTE
        """
        return self.sampler.execute(self.get_portamento_mode_cmd, ())

    def get_portamento_time(self):
        """Get Portamento Time

        Returns:
            BYTE
        """
        return self.sampler.execute(self.get_portamento_time_cmd, ())

    def set_lfo_rate(self, arg0, arg1):
        """Set LFO Rate. <Data1> = rate
        """
        return self.sampler.execute(self.set_lfo_rate_cmd, (arg0, arg1, ))

    def set_lfo_delay(self, arg0, arg1):
        """Set LFO Delay. <Data1> = delay
        """
        return self.sampler.execute(self.set_lfo_delay_cmd, (arg0, arg1, ))

    def set_lfo_depth(self, arg0, arg1):
        """Set LFO Depth. <Data1> = depth
        """
        return self.sampler.execute(self.set_lfo_depth_cmd, (arg0, arg1, ))

    def set_lfo_waveform(self, arg0, arg1):
        """Set LFO Waveform. <Data1> = waveform, where: 0=SINE, 1=TRIANGLE, 2=SQUARE, 3=SQUARE+, 4=SQUARE, 5=SAW BI, 6=SAW UP, 7=SAW DOWN, 8=RANDOM
        """
        return self.sampler.execute(self.set_lfo_waveform_cmd, (arg0, arg1, ))

    def set_lfo_sync(self, arg0, arg1):
        """Set LFO Sync. <Data1> = (0=OFF, 1=ON). (LFO1 only)
        """
        return self.sampler.execute(self.set_lfo_sync_cmd, (arg0, arg1, ))

    def set_lfo_retrigger(self, arg0, arg1):
        """Set LFO Re-trigger. <Data1> = (0=OFF, 1=ON). (LFO2 only)
        """
        return self.sampler.execute(self.set_lfo_retrigger_cmd, (arg0, arg1, ))

    def set_rate_modulation_source(self, arg0, arg1):
        """Set Rate Mod Source <Data1> = Modulation Source. (see Table 15)
        """
        return self.sampler.execute(self.set_rate_modulation_source_cmd, (arg0, arg1, ))

    def get_lfo_sync(self, arg0):
        """Get LFO Sync (LFO1 only)
        """
        return self.sampler.execute(self.get_lfo_sync_cmd, (arg0, ))

    def get_lfo_retrigger(self, arg0):
        """Get LFO Re-trigger (LFO2 only)
        """
        return self.sampler.execute(self.get_lfo_retrigger_cmd, (arg0, ))

    def get_rate_modulation_source(self, arg0):
        """Get Rate Mod Source

        Returns:
            BYTE
        """
        return self.sampler.execute(self.get_rate_modulation_source_cmd, (arg0, ))

    def get_rate_modulation_value(self, arg0):
        """Get Rate Mod Value

        Returns:
            SBYTE
        """
        return self.sampler.execute(self.get_rate_modulation_value_cmd, (arg0, ))

    def get_delay_modulation_source(self, arg0):
        """Get Delay Mod Source

        Returns:
            BYTE
        """
        return self.sampler.execute(self.get_delay_modulation_source_cmd, (arg0, ))

    def get_delay_modulation_value(self, arg0):
        """Get Delay Mod Value

        Returns:
            SBYTE
        """
        return self.sampler.execute(self.get_delay_modulation_value_cmd, (arg0, ))

    def get_depth_modulation_source(self, arg0):
        """Get Depth Mod Source

        Returns:
            BYTE
        """
        return self.sampler.execute(self.get_depth_modulation_source_cmd, (arg0, ))

    def get_depth_modulation_value(self, arg0):
        """Get Depth Mod Value

        Returns:
            SBYTE
        """
        return self.sampler.execute(self.get_depth_modulation_value_cmd, (arg0, ))

    def get_modwheel(self, arg0):
        """Get Modwheel (LFO1 only)

        Returns:
            BYTE
        """
        return self.sampler.execute(self.get_modwheel_cmd, (arg0, ))

    def get_aftertouch(self, arg0):
        """Get Aftertouch (LFO1 only)

        Returns:
            BYTE
        """
        return self.sampler.execute(self.get_aftertouch_cmd, (arg0, ))

    def get_midi_clock_sync_enabled(self):
        """Get MIDI Clock Sync Enable (LFO2 only)

        Returns:
            BOOL
        """
        return self.sampler.execute(self.get_midi_clock_sync_enabled_cmd, ())

    def get_midi_clock_sync_division(self, arg0):
        """Get MIDI Clock Sync Division (LFO2 only)

        Returns:
            BYTE
        """
        return self.sampler.execute(self.get_midi_clock_sync_division_cmd, (arg0, ))

    def set_pitch_modulation_source(self, arg0, arg1):
        """Set Pitch Mod Source. <Data1> = Modulation Source. (see Table 15)
        """
        return self.sampler.execute(self.set_pitch_modulation_source_cmd, (arg0, arg1, ))

    def set_filter_modulation_source(self, arg0, arg1):
        """Set Filter Mod Input Source. <Data1> = Modulation Source. (see Table 15)
        """
        return self.sampler.execute(self.set_filter_modulation_source_cmd, (arg0, arg1, ))

    def get_pitch_modulation_source(self, arg0):
        """Get Pitch Mod Source. <Data1> = Pitch Mod (1 or 2)

        Returns:
            BYTE
        """
        return self.sampler.execute(self.get_pitch_modulation_source_cmd, (arg0, ))

    def get_filter_modulation_source(self, arg0):
        """Get Filter Mod Input Source. <Data1> = Mod Input (1, 2 or 3)

        Returns:
            BYTE
        """
        return self.sampler.execute(self.get_filter_modulation_source_cmd, (arg0, ))

    def get_handles_names(self):
        """Get list of program handles and names

        Returns:
            HANDLENAMEARRAY
        """
        handle_names = []
        names = self.get_names()
        for i in range(len(names)):
            handle_names.extend([i, names[i]])
        return handle_names




import ConfigParser
import os.path, sys
from optparse import OptionParser

def create_option_parser(usage):
    parser = OptionParser(usage=usage)
    config = get_config()
    parser.add_option("-t", nargs=1, dest="sampler_type",
      help="Type of sampler (z48/mpc4k/s56k)", default=get_value(config, 'sampler', 'type'))
    parser.add_option("-c", nargs=1, dest="connector",
      help="Type of connector", default=get_value(config, 'sampler', 'connector'))

    return parser

def get_value(config, default, prop_name):
    section_override = os.path.basename(sys.argv[0])
    try:
        return config.get(section_override, prop_name)
    except (ConfigParser.NoSectionError, ConfigParser.NoOptionError):
        return config.get(default, prop_name)

def get_config(ini_file=os.path.expanduser('~/.aksy/aksy.ini')):
    cfg = ConfigParser.ConfigParser(defaults={ 'port': '6575', 'host': 'localhost', 'basedir': os.path.expanduser('~') })
    cfg.add_section('sampler')
    cfg.set('sampler', 'type', 'z48')
    cfg.set('sampler', 'connector', 'usb')
    cfg.add_section('osc')
    cfg.add_section('osc_server')
    cfg.add_section('osc_client')
    cfg.add_section('logging')
    cfg.set('logging', 'level', 'INFO')

    cfg.read(ini_file)
    return cfg





""" Python equivalent of akai section disktools

Methods to manipulate the sampler's filesystem
"""

__author__ =  'Walco van Loon'
__version__ =  '0.2'

from aksy.devices.akai.sysex import Command

import aksy.devices.akai.sysex_types

class Disktools:
    def __init__(self, s56k):
        self.sampler = s56k
        self.update_disklist_cmd = Command('^', '\x10\x01', 'disktools', 'update_disklist', (), (), aksy.devices.akai.sysex_types.S56K_USERREF)
        self.select_disk_cmd = Command('^', '\x10\x02', 'disktools', 'select_disk', (aksy.devices.akai.sysex_types.CWORD,), (), aksy.devices.akai.sysex_types.S56K_USERREF)
        self.test_disk_cmd = Command('^', '\x10\x03', 'disktools', 'test_disk', (aksy.devices.akai.sysex_types.CWORD,), (), aksy.devices.akai.sysex_types.S56K_USERREF)
        self.get_no_disks_cmd = Command('^', '\x10\x04', 'disktools', 'get_no_disks', (), (aksy.devices.akai.sysex_types.BYTE,), aksy.devices.akai.sysex_types.S56K_USERREF)
        self.get_disklist_cmd = Command('^', '\x10\x05', 'disktools', 'get_disklist', (), (aksy.devices.akai.sysex_types.S56KDISKLIST,), aksy.devices.akai.sysex_types.S56K_USERREF)
        self.get_curr_path_cmd = Command('^', '\x10\x09', 'disktools', 'get_curr_path', (), (aksy.devices.akai.sysex_types.STRINGARRAY,), aksy.devices.akai.sysex_types.S56K_USERREF)
        self.eject_disk_cmd = Command('^', '\x10\x0D', 'disktools', 'eject_disk', (aksy.devices.akai.sysex_types.CWORD,), (), aksy.devices.akai.sysex_types.S56K_USERREF)
        self.get_no_folders_cmd = Command('^', '\x10\x10', 'disktools', 'get_no_folders', (), (aksy.devices.akai.sysex_types.CWORD,), aksy.devices.akai.sysex_types.S56K_USERREF)
        self.get_folder_names_cmd = Command('^', '\x10\x12', 'disktools', 'get_folder_names', (), (aksy.devices.akai.sysex_types.STRINGARRAY,), aksy.devices.akai.sysex_types.S56K_USERREF)
        self.open_folder_cmd = Command('^', '\x10\x13', 'disktools', 'open_folder', (aksy.devices.akai.sysex_types.STRING,), (), aksy.devices.akai.sysex_types.S56K_USERREF)
        self.close_folder_cmd = Command('^', '\x10\x14', 'disktools', 'close_folder', (), (), aksy.devices.akai.sysex_types.S56K_USERREF)
        self.load_folder_cmd = Command('^', '\x10\x15', 'disktools', 'load_folder', (aksy.devices.akai.sysex_types.STRING,), (), aksy.devices.akai.sysex_types.S56K_USERREF)
        self.create_folder_cmd = Command('^', '\x10\x16', 'disktools', 'create_folder', (aksy.devices.akai.sysex_types.STRING,), (), aksy.devices.akai.sysex_types.S56K_USERREF)
        self.delete_folder_cmd = Command('^', '\x10\x17', 'disktools', 'delete_folder', (aksy.devices.akai.sysex_types.STRING,), (), aksy.devices.akai.sysex_types.S56K_USERREF)
        self.rename_folder_cmd = Command('^', '\x10\x18', 'disktools', 'rename_folder', (aksy.devices.akai.sysex_types.STRING, aksy.devices.akai.sysex_types.STRING), (), aksy.devices.akai.sysex_types.S56K_USERREF)
        self.get_no_files_cmd = Command('^', '\x10\x20', 'disktools', 'get_no_files', (), (aksy.devices.akai.sysex_types.CWORD,), aksy.devices.akai.sysex_types.S56K_USERREF)
        self.get_filenames_cmd = Command('^', '\x10\x22', 'disktools', 'get_filenames', (), (aksy.devices.akai.sysex_types.STRINGARRAY,), aksy.devices.akai.sysex_types.S56K_USERREF)
        self.rename_file_cmd = Command('^', '\x10\x28', 'disktools', 'rename_file', (aksy.devices.akai.sysex_types.STRING, aksy.devices.akai.sysex_types.STRING), (), aksy.devices.akai.sysex_types.S56K_USERREF)
        self.delete_file_cmd = Command('^', '\x10\x29', 'disktools', 'delete_file', (aksy.devices.akai.sysex_types.STRING,), (), aksy.devices.akai.sysex_types.S56K_USERREF)
        self.load_file_cmd = Command('^', '\x10\x2A', 'disktools', 'load_file', (aksy.devices.akai.sysex_types.STRING,), (), aksy.devices.akai.sysex_types.S56K_USERREF)
        self.load_file_and_deps_cmd = Command('^', '\x10\x2B', 'disktools', 'load_file_and_deps', (aksy.devices.akai.sysex_types.STRING,), (), aksy.devices.akai.sysex_types.S56K_USERREF)
        self.save_cmd = Command('^', '\x10\x2C', 'disktools', 'save', (aksy.devices.akai.sysex_types.DWORD, aksy.devices.akai.sysex_types.BYTE, aksy.devices.akai.sysex_types.BOOL, aksy.devices.akai.sysex_types.BYTE), (), aksy.devices.akai.sysex_types.S56K_USERREF)
        self.save_all_cmd = Command('^', '\x10\x2D', 'disktools', 'save_all', (aksy.devices.akai.sysex_types.BYTE, aksy.devices.akai.sysex_types.BOOL), (), aksy.devices.akai.sysex_types.S56K_USERREF)

    def update_disklist(self):
        """Update the list of disks connected
        """
        return self.sampler.execute(self.update_disklist_cmd, ())

    def select_disk(self, arg0):
        """Select Disk <Data1> = Disk Handle
        """
        return self.sampler.execute(self.select_disk_cmd, (arg0, ))

    def test_disk(self, arg0):
        """Test if the disk is valid <Data1> = Disk Handle
        """
        return self.sampler.execute(self.test_disk_cmd, (arg0, ))

    def get_no_disks(self):
        """Get the number of disks connected

        Returns:
            BYTE
        """
        return self.sampler.execute(self.get_no_disks_cmd, ())

    def get_disklist(self):
        """Get list of all connected disks

        Returns:
            DISKLIST
        """
        return self.sampler.execute(self.get_disklist_cmd, ())

    def get_curr_path(self):
        """Get current path of current disk

        Returns:
            STRING
        """
        return self.sampler.execute(self.get_curr_path_cmd, ())

    def eject_disk(self, arg0):
        """Eject Disk <Data1> = Disk Handle
        """
        return self.sampler.execute(self.eject_disk_cmd, (arg0, ))

    def get_no_folders(self):
        """Get number of sub-folders in the current folder.

        Returns:
            CWORD
        """
        return self.sampler.execute(self.get_no_folders_cmd, ())

    def get_folder_names(self):
        """Get the names of all of the sub-folders in the current folder.

        Returns:
            STRINGARRAY
        """
        return self.sampler.execute(self.get_folder_names_cmd, ())

    def open_folder(self, arg0):
        """Open Folder. This sets the current folder to be the requested one. (If <Data1> = 0, the root folder will be selected.)
        """
        return self.sampler.execute(self.open_folder_cmd, (arg0, ))

    def close_folder(self):
        """Move up one level in the folder hierarchy. If this is used on a root folder, an ERROR confirmation message will be returned.
        """
        return self.sampler.execute(self.close_folder_cmd, ())

    def load_folder(self, arg0):
        """Load Folder: the selected folder, and all its contents (including subfolders)
        """
        return self.sampler.execute(self.load_folder_cmd, (arg0, ))

    def create_folder(self, arg0):
        """Create Folder: Creates a sub-folder in the currently selected folder.
        """
        return self.sampler.execute(self.create_folder_cmd, (arg0, ))

    def delete_folder(self, arg0):
        """Delete Sub-Folder.
        """
        return self.sampler.execute(self.delete_folder_cmd, (arg0, ))

    def rename_folder(self, arg0, arg1):
        """Rename Folder: <Data1> = name of folder to rename
        """
        return self.sampler.execute(self.rename_folder_cmd, (arg0, arg1, ))

    def get_no_files(self):
        """Get number of files in the current folder.

        Returns:
            CWORD
        """
        return self.sampler.execute(self.get_no_files_cmd, ())

    def get_filenames(self):
        """Get the names of all of the files in the current folder.

        Returns:
            STRINGARRAY
        """
        return self.sampler.execute(self.get_filenames_cmd, ())

    def rename_file(self, arg0, arg1):
        """Rename File
        """
        return self.sampler.execute(self.rename_file_cmd, (arg0, arg1, ))

    def delete_file(self, arg0):
        """Delete File. <Data1> = name of file to delete.
        """
        return self.sampler.execute(self.delete_file_cmd, (arg0, ))

    def load_file(self, arg0):
        """Load File <Data1> = name of file to load.
        """
        return self.sampler.execute(self.load_file_cmd, (arg0, ))

    def load_file_and_deps(self, arg0):
        """Load File <Data1> = name of file to load. Will load the dependents as well
        """
        return self.sampler.execute(self.load_file_and_deps_cmd, (arg0, ))

    def save(self, arg0, arg1, arg2, arg3):
        """Save Memory Item to Disk <Data1> = Handle of Memory Item <Data2> = Type = (1=Multi; 2=Program; 3=Sample; 4=SMF) <Data3> = (0=Skip if file exists; 1=Overwrite existing files) <Data4> = (0=Don't save children; 1=Save Children)
        """
        return self.sampler.execute(self.save_cmd, (arg0, arg1, arg2, arg3, ))

    def save_all(self, arg0, arg1):
        """Save All Memory Items to Disk <Data1> = Type = (0=All; 1=Multi; 2=Program; 3=Sample; 4=SMF) <Data2> = (0=Skip if file exists; 1=Overwrite existing files)
        """
        return self.sampler.execute(self.save_all_cmd, (arg0, arg1, ))




import re

RE_MULTI = re.compile("\.akm$", re.IGNORECASE)
RE_PROGRAM = re.compile("\.(akp|pgm)$", re.IGNORECASE)
RE_SAMPLE = re.compile("\.(wav|aiff?)$", re.IGNORECASE)
RE_SONG = re.compile("\.mid$", re.IGNORECASE)
# the entire printable ascii range
RE_WORD = re.compile("[ -~]+$", re.IGNORECASE)
RE_FILE = re.compile("\.(akm|akp|pgm|wav|aiff?|mid)$", re.IGNORECASE)


def is_file_type_supported(supported_file_types, filename):
    for file_type in supported_file_types:
        if filename.lower().endswith(file_type):
            return True
    return False

def is_dirpath(path):
    return path.count('.') == 0 or not is_file(path)

def is_file(path):
    return RE_FILE.search(path) is not None

def is_sample(path):
    return RE_SAMPLE.search(path) is not None

def is_multi(path):
    return RE_MULTI.search(path) is not None

def is_program(path):
    return RE_PROGRAM.search(path) is not None

def is_song(path):
    return RE_SONG.search(path) is not None

def is_valid_name(name):
    return RE_WORD.match(name) is not None


NOTE_NAMES = 'a', 'b', 'bes', 'c', 'cis', 'd' 'es', 'e', 'f', 'fis', 'g', 'gis'

class Envelope:
    @staticmethod
    def create_default():
        env = Envelope()
        env.attack = 0
        env.decay = 50
        env.release = 15
        env.sustain = 100
        env.velo_to_attack = 0
        env.velo_on_to_release = 0
        env.velo_off_to_release = 0
        env.keyscale = 0
        return env

class AmpEnvelope(Envelope):
   pass

class FilterEnvelope(Envelope):
    @staticmethod
    def create_default():
        env = Envelope.create_default()
        env.filter_env_depth = 0
        return env
        
class AuxEnvelope:
    @staticmethod
    def create_default():
        env = Envelope()
        env.rate1 = 0
        env.rate2 = 50
        env.rate3 = 50
        env.rate4 = 15 
        env.level1 = 100 
        env.level2 = 100 
        env.level3 = 100 
        env.level4 = 0 
        env.velo_to_rate1 = 0
        env.kb_to_r2_r4 = 0
        env.velo_on_to_r4 = 0
        env.velo_off_to_r4 = 0
        env.velo_to_out_level = 0
        return env

class Filter:
    LP2P = 0
    LP4P = 1 
    LP4PPLUS = 2
    BP2P = 3
    BP4P = 4
    BP2PPLUS = 5
    HP1 = 6
    HP2 = 7
    HP8PLUS = 8
    LO_HI = 9
    LO_BAND = 10
    BAND_HI = 11
    NOTCH1 = 12
    NOTCH2 = 13
    NOTCH3 = 14
    NOTCH_WIDE = 15
    NOTCH_BI = 16
    PEAK1 = 17
    PEAK1 = 18
    PEAK1 = 19
    PEAK_WIDE = 20
    PEAK_BI = 21 
    PHASER1 = 22 
    PHASER2 = 23 
    PHASER_BI = 24 
    VOWEL = 25 

    HEADROOM_0db = 0
    HEADROOM_6db = 1
    HEADROOM_12db = 2
    HEADROOM_18db = 3
    HEADROOM_24db = 4
    HEADROOM_30db = 5
    
    @staticmethod
    def create_default():
        filter = Filter()
        filter.mode = 0 # 2 pole LP
        filter.cutoff = 100
        filter.resonance = 0
        filter.kb_track = 0
        filter.mod1_input = 0
        filter.mod2_input = 0
        filter.mod3_input = 0
        filter.headroom = 0
        return filter

class LFO:
    """ 
    >>> lfo = LFO(waveform=LFO.SAW_BI, rate_mod=36)
    """
    SINE = 0  
    TRIANGLE = 1
    SQUARE = 2
    SQUARE_PLUS = 3
    SQUARE_MINUS = 4
    SAW_BI = 5
    SAW_UP = 6
    SAW_DOWN = 7
    RANDOM = 8

    @staticmethod
    def create_default():
        lfo = LFO()
        lfo.waveform = LFO.TRIANGLE 
        lfo.rate = 43
        lfo.delay = 0
        lfo.depth = 0
        lfo.rate_mod = 0
        lfo.delay_mod = 0
        lfo.depth_mod = 0
        return lfo
 
class LFO1(LFO):
    @staticmethod
    def create_default(self):
        lfo = LFO.create_default()
        lfo.lfo_sync = 0
        lfo.phase = 0 # from sysex docs 
        lfo.lfo_shift   = 0 # from sysex docs 
        lfo.lfo_mid_clock_div = 0 # from sysex docs
        lfo.modwheel = 15
        lfo.aftertouch = 0
        return lfo
    
class LFO2(LFO):
    @staticmethod
    def create_default():
        lfo = LFO.create_default()
        lfo.lfo_retrigger = 0
        return lfo
        
class Modulation:
    """
    Contains modulation routings
    """

    NO_SOURCE = 0  
    MODWHEEL = 1
    BEND = 2 
    AFTERTOUCH = 3
    EXTERNAL = 4
    VELOCITY = 5
    KEYBOARD = 6 
    LFO1 = 7
    LFO2 = 8
    AMP_ENV = 9
    FILT_ENV = 10
    AUX_ENV = 11  
    MIDI_MODWHEEL = 12
    MIDI_BEND = 13
    MIDI_EXTERNAL = 14 
    
    @staticmethod
    def create_default():
        mod = Modulation()
        mod.amp_src1 = 6
        mod.amp_src2 = 3
        mod.pan_src1 = 8
        mod.pan_src2 = 6
        mod.pan_src3 = 1
        mod.lfo1_rate_src = 6
        mod.lfo1_delay_src = 6
        mod.lfo1_depth_src = 6
        mod.lfo2_rate_src = 0
        mod.lfo2_delay_src = 0
        mod.lfo2_depth_src = 0
        # keygroup mod
        mod.pitch1_src = 7
        mod.pitch2_src = 11 
        mod.amp_src = 11 
        mod.filter_input1 = 5 
        mod.filter_input2 = 8 
        mod.filter_input3 = 9 
        return mod

class Program:
    def __init__(self, keygroups):
        self.keygroups = keygroups
        
    @staticmethod
    def create_default(keygroups):
        program = Program(keygroups)
        program.midi_prog_no = 0
        program.no_keygrps = 1 
        program.loudness = 85
        program.amp_mod1 = 0
        program.amp_mod2 = 0
        program.pan_mod1 = 0
        program.pan_mod2 = 0
        program.pan_mod3 = 0
        program.velo_sens = 25
        program.semi = 0 
        program.fine = 0 
        program.a_detune = 0
        program.ais_detune = 0
        program.b_detune = 0
        program.bes_detune = 0
        program.c_detune = 0
        program.cis_detune = 0
        program.des_detune = 0
        program.e_detune = 0
        program.f_detune = 0
        program.fis_detune = 0
        program.g_detune = 0
        program.gis_detune = 0
        program.pitchbend_up = 2
        program.pitchbend_down = 2
        program.bend_mode = 0 # 1: held
        program.aftertouch = 0
        program.lfos = (LFO1(), LFO2(),)
        program.mod = Modulation()
        return program

class Keygroup:
    def __init__(self, zones):
        self.zones = zones
        
    @staticmethod
    def create_default(zones):
        kg = Keygroup(zones)
        kg.low_note=21 
        kg.high_note=127 
        kg.semi=0
        kg.fine=0 
        kg.override_fx=0
        kg.fx_send=0
        kg.pitch_mod1=100
        kg.pitch_mod2=0
        kg.amp_mod=0
        kg.zone_xfade=0
        kg.mute_group=0
        kg.envelopes = (AmpEnvelope(), FilterEnvelope(), AuxEnvelope())
        kg.filter = Filter()
        return kg

class Zone:
    @staticmethod
    def create_default():
        zone = Zone()
        # XXX: 2 bytes missing from description
        zone.samplename = '' # no sample assigned 
        zone.low_vel = 0
        zone.high_vel = 127
        zone.fine = 0
        zone.semi = 0
        zone.filter = 0
        zone.pan = 0
        zone.playback = 4 # as sample
        zone.output = 0 # multi
        zone.level = 0
        zone.kb_track = 1
        zone.velo_start = 0
        return zone
        
class Chunk:
    def __init__(self, name, bytes, length):
        self.name = name
        self.bytes = bytes
        self.length = length
    def get_length(self):
        return self.length
    def get_content_length(self):
        return self.length - 8
    
    def __repr__(self):
        return "<Chunk %s, length=%i, bytes=%s>" % (repr(self.name), self.length, repr(self.bytes))
        


import gtk.glade

import os.path

from pkg_resources import resource_string
glade_str = resource_string('aksui', 'ak.py.glade')

def get_glade_xml(root):
    return gtk.glade.xml_new_from_buffer(glade_str, len(glade_str), root)

class Base(object):
    def __init__(self, samplerobject, editor_root):
        self.widgets = { }
        if samplerobject:
            self.samplerobject = samplerobject
            self.s = self.samplerobject.s
            self.index = self.samplerobject.index
        else:
            self.samplerobject = None

        self.updating = False
        #print editor_root
        self.xml = get_glade_xml(editor_root)
        self.editor = self.xml.get_widget(editor_root)
        self.finish_editors()

    def finish_editors(self):
        self.xml.signal_autoconnect(self)

    def update(self):
        if self.samplerobject:
            self.updating = True
            self.samplerobject.update()
            tf = {0:False, 1:True}
            samplerobject = self.samplerobject
            xml = self.xml

            for attr in samplerobject.attrs:
                combo = xml.get_widget("combo_" + attr)
                entry = xml.get_widget("entry_" + attr)
                spin = xml.get_widget("spin_" + attr)
                toggle = xml.get_widget("toggle_" + attr)
                hscale = xml.get_widget("hscale_" + attr)

                val = getattr(samplerobject,attr)

                try:
                    if combo != None:
                        if attr != "sample":
                            combo.set_active(val)
                    elif entry != None:
                        entry.set_text(str(val))
                    elif toggle != None:
                        toggle.set_active(tf[val])
                    elif spin != None:
                        spin.set_value(val)
                    elif hscale != None:
                        hscale.set_value(val)
                except Exception, ex:
                    print 'attr',attr,val,ex

            self.updating = False

    def on_param_changed(self, widget):
        if self.samplerobject:
            if not self.updating:
                if type(widget) is gtk.HScale:
                    attrname, attrval = widget.name[7:], widget.get_value()
                    self.samplerobject.set(attrname, int(attrval))
                elif type(widget) is gtk.Entry:
                    attrname, attrval = widget.name[6:], widget.get_text()
                    self.samplerobject.set(attrname, attrval)
                elif type(widget) is gtk.ComboBox:
                    attrname, attrval = widget.name[6:], widget.get_active()
                    self.samplerobject.set(attrname, attrval)
                elif type(widget) is gtk.ToggleButton:
                    attrname, attrval = widget.name[7:], widget.get_active()
                    self.samplerobject.set(attrname, attrval)
                elif type(widget) is gtk.SpinButton:
                    attrname, attrval = widget.name[5:], widget.get_value()
                    self.samplerobject.set(attrname, int(attrval))

    def pop_in(self, container_widget_name, child_widget_name):
        setattr(self,container_widget_name,self.xml.get_widget(container_widget_name))
        setattr(self,child_widget_name,self.xml.get_widget(child_widget_name))
        w = getattr(self,container_widget_name)
        ch = getattr(self,child_widget_name)

        if type(w) is gtk.Notebook:
            w.append_page(ch)
        elif type(w) is gtk.Box:
            w.pack_start(ch)
        elif type(w) is gtk.Container:
            # default
            w.add(ch)

    def get_widget(self, name):
        return self.xml.get_widget(name)
    
    def show_all(self):
        self.editor.show_all()

    def set_samplerobject(self, so):
        self.samplerobject = so
        self.update()

    def __getattribute__(self, name):
        if name.startswith('w_') and not name in self.widgets:
            setattr(self, name, self.xml.get_widget(name[2:]))
            self.widgets[name] = object.__getattribute__(self, name)

        return object.__getattribute__(self, name)



import struct, logging, errno
from aksy.devices.akai import sysex_types, model
from aksy.devices.akai.sysex_types import START_SYSEX, END_SYSEX

AKAI_ID = '\x47'
Z48_ID = '\x5f'
S56K_ID = '\x5e'

REPLY_ID_OK = '\x4f'
REPLY_ID_DONE = '\x44'
REPLY_ID_REPLY = '\x52'
REPLY_ID_ERROR = '\x45'

log = logging.getLogger("aksy")

class Command:
    """Represents a system exclusive command.
    """
    def __init__(self, device_id, cmd_id, section, name, arg_types,
            reply_spec, userref_type=sysex_types.USERREF):
        self.device_id = device_id
        self.id = cmd_id
        self.section = section
        self.name = name
        self.arg_types = []
        self.reply_spec = reply_spec
        self.userref_type = userref_type

        for arg_type in arg_types:
            if arg_type is not None:
                self.arg_types.append(arg_type)

    def create_arg_bytes(self, args):
        """Returns the sysex byte sequence for the command arg data -
        """
        bytes = []
        for sysex_type, arg in zip(self.arg_types, args):
            bytes.append(sysex_type.encode(arg))

        if len(bytes) == 0:
            return None
        else:
            return bytes

class Request:
    """ Encapsulates a sysex request
    command: the command to execute
    """
    def __init__(self, command, args, request_id=0):
        self.command = command
        self.args = args
        self.bytes = self._create_bytes(command, args, request_id)
        
    def _create_bytes(self, command, args, request_id):
        bytes = self._create_start_bytes(command, request_id)
        bytes.append(command.id)
        data = command.create_arg_bytes(args)
        if data is not None:
            bytes.extend(data)
        bytes.append(END_SYSEX)

        return ''.join(bytes)

    def _create_start_bytes(self, command, request_id):
        bytes = [START_SYSEX, AKAI_ID]
        bytes.append(command.device_id)
        bytes.append(command.userref_type.encode(request_id))
        return bytes
        
    def get_bytes(self):
        return self.bytes
    
    def __repr__(self):
        return repr([ "%02x" %byte for byte in struct.unpack(str(len(self.bytes)) + 'B', 
                                                             self.bytes)])
class AlternativeRequest(Request):
    SECTION_SAMPLE = '\x60'
    SECTION_KEYGROUP = '\x61'
    SECTION_PROGRAM = '\x62'
    SECTION_MULTI = '\x63'
    SECTION_MULTIFX = '\x64'
    SECTION_SONG = '\x65'

    BASE_SECTION_SAMPLE = '\x1c'
    BASE_SECTION_KEYGROUP = '\x0c'
    BASE_SECTION_PROGRAM = '\x14'
    BASE_SECTION_MULTI = '\x18'
    BASE_SECTION_MULTIFX = '\x25'
    BASE_SECTION_SONG = '\x28'

    BASE_ALT_SECTION_MAP = {
        BASE_SECTION_SAMPLE : SECTION_SAMPLE,
        BASE_SECTION_KEYGROUP : SECTION_KEYGROUP,
        BASE_SECTION_PROGRAM : SECTION_PROGRAM,
        BASE_SECTION_MULTI : SECTION_MULTI,
        BASE_SECTION_MULTIFX : SECTION_MULTIFX,
        BASE_SECTION_SONG : SECTION_SONG
    }

    def __init__(self, handle, commands, args, index=None, request_id=0):
        bytes = self._create_start_bytes(commands[0], request_id)
        no_sections = index is None and 4 or 8
        # TODO: this is z48 specific
        alt_section_id, offset = self.find_section_ids(commands[0].id, no_sections)
        bytes.append(alt_section_id)
        bytes.append(offset)
        bytes.append(sysex_types.DWORD.encode(handle))
        if index is not None:
            bytes.append(sysex_types.BYTE.encode(index))
            
        for i, command in enumerate(commands):
            if len(command.arg_types) > 0:
                data = ''.join(command.create_arg_bytes(args[i]))
                bytes.append(sysex_types.BYTE.encode(len(data) + 1))
            else:
                data = []
                bytes.append(sysex_types.BYTE.encode(1))

            bytes.append(command.id[1])
            bytes.extend(data)
        
                 
        bytes.append(END_SYSEX)
        self.bytes = ''.join(bytes)
        
    def find_section_ids(self, command_id, no_sections):
        section_id = sysex_types.BYTE.decode(command_id[:1], False)
        for i in range(no_sections):
            base_section =  sysex_types.BYTE.encode(section_id - i)
            alt_section = self.BASE_ALT_SECTION_MAP.get(base_section, None) 
            if alt_section is not None:
                return alt_section, sysex_types.BYTE.encode(i)
        
        raise model.SamplerException("No alternative operations defined for %s" % 
                                    repr(command_id[:1]))
        
    
class Reply:
    """ Encapsulates a sysex reply
    """
    def __init__(self, bytes, command, alt_request=False):
        self.bytes = bytes
        self.command = command
        self.request_id  = 0
        self.alt_request = alt_request
        self.return_value = self._parse()

    def get_request_id(self):
        return self.request_id

    def get_return_value(self):
        return self.return_value

    def _parse(self):
        """ Parses the command sequence
        """

        if self.bytes[0] != START_SYSEX or self.bytes[-1] != END_SYSEX:
            raise ParseException("Invalid system exclusive string received")
        # keep alive message
        if len(self.bytes) == 2:
            return None
        # TODO: dispatching on z48id, userref and command
        i = 2   # skip start sysex, vendor id
        i += len(self.command.device_id)
        len_userref, self.request_id = self.command.userref_type.decode(self.bytes[i:])
        i += len_userref
        reply_id = self.bytes[i]
        i +=  1 # skip past the reply code
        command = self.bytes[i:i+2]
        i += len(self.command.id) # skip past the command id (section, item, optional subcmd)
        if reply_id == REPLY_ID_OK:
            return None
        elif reply_id == REPLY_ID_DONE:
            return None
        elif reply_id == REPLY_ID_ERROR:
            byte1, byte2 = struct.unpack('2B', self.bytes[i:i+2])
            code = (byte2 << 7) + byte1
            raise _create_exception(
                errors.get(code, "Unknown"), code)
        elif reply_id == REPLY_ID_REPLY:
            # continue
            pass
        else:
            raise ParseException("Unknown reply type: %02x" % struct.unpack('b', reply_id))

        if not self.alt_request and self.command.id[:2] != command:
            raise ParseException(
                'Parsing the wrong reply for command %02x %02x'
                    % struct.unpack('2B', self.command.id[:2]))
        if self.command.reply_spec is None:
            return sysex_types.TYPED_COMPOSITE.decode(self.bytes[i:])[1]
        else:
            return sysex_types.CompositeType(self.command.reply_spec).decode(self.bytes[i:])[1]

class ParseException(Exception):
    """ Exception raised when parsing system exclusive fails
    """
    pass

def byte_repr(bytes):
    return repr([ "%02x" %byte for byte in struct.unpack(str(len(bytes)) + 'B', bytes)])

def repr_bytes(bytes):
    return  ''.join([struct.pack('1B', int(byte, 16)) for byte in bytes])

def _to_string(ordvalues):
    """Method to quickly convert to a string
    >>> ordvalues = (90, 52, 56, 32, 38, 32, 77, 80, 67, 52, 75)
    >>> _to_string(ordvalues)
    'Z48 & MPC4K'
    """
    return ''.join([chr(value) for value in ordvalues])

def _create_exception(msg, code):
    if code == 0x101 or code == 0x203:
        return IOError(errno.ENOENT, msg)
    if code == 0x0:
        return NotImplementedError(msg)
        
    return model.SamplerException(msg, code)

errors = {
    0x00:"The <Section> <Item> supplied are not supported",
    0x01:"Checksum invalid",
    0x02:"Unknown error",
    0x03:"Invalid message format",
    0x04:"Parameter out of range",
    0x05:"Operation is pending",
    0x80:"Unknown system error",
    0x81:"Operation had no effect",
    0x82:"Fatal error",
    0x83:"CPU memory is full",
    0x84:"WAVE memory is full",
    0x100:"Unknown item error",
    0x101:"Item not found",
    0x102:"Item in use",
    0x103:"Invalid item handle",
    0x104:"Invalid item name",
    0x105:"Maximum number of items of a particular type reached",
    0x120:"Keygroup not found",
    0x180:"Unknown disk error",
    0x181:"No Disks",
    0x182:"Disk is invalid",
    0x183:"Load error",
    0x184:"Create error",
    0x185:"Directory not empty",
    0x186:"Delete error",
    0x187:"Disk is write-protected",
    0x188:"Disk is not writable",
    0x189:"Disk full",
    0x18A:"Disk abort",
    0x200:"Unknown file error",
    0x201:"File format is not supported",
    0x202:"WAV format is incorrect",
    0x203:"File not found",
    0x204:"File already exists",
}



import modelutils

# list of mpc pad -> notes, left -> right = 1 -> 16
mpcpads = [
            37,36,42,82,40,38,46,44,48,47,45,43,49,55,51,53, # a
            54,69,81,80,65,66,76,77,56,62,63,64,73,74,71,39, # b
            52,57,58,59,60,61,67,68,70,72,75,78,79,35,41,50, # c
            83,84,85,86,87,88,89,90,91,92,93,94,95,96,97,98  # d
            ]

mpc_banks_gm = [
            49,55,51,53, 
            48,47,45,43,
            40,38,46,44,
            37,36,42,82,
            73,74,71,39, 
            65,66,76,77,
            56,62,63,64,
            54,69,81,80,
            79,35,41,50, 
            70,72,75,78,
            60,61,67,68,
            52,57,58,59,
            95,96,97,98, 
            91,92,93,94,
            87,88,89,90,
            83,84,85,86,
            ]
mpc_banks_chromatic = [
            48,49,50,51, 
            44,45,46,47,
            40,41,42,43,
            36,37,38,39,
            64,65,66,67,
            60,61,62,63,
            56,57,58,59,
            52,53,54,55,
            80,81,82,83,
            76,77,78,79,
            72,73,74,75,
            68,69,70,71,
            96,97,98,99,
            92,93,94,95,
            88,89,90,91,
            84,85,86,87,
            ]

notes = ['C','C#','D','Eb','E','F','F#','G','G#','A','Bb','B']
octaves = [i for i in range(-2, 9)]
octaves_snap = [i for i in range(0, 127, 12)]
midinotes = { }

i = 0

for oct in octaves:
    for note in notes:
        if i > 127:
            break
        if oct >= 0 and len(note) == 1:
            n = note + ' ' + str(oct)
        else:
            n = note + str(oct)
        midinotes[i] = n
        i = i + 1        

mpcpadsmodel = modelutils.get_model_from_list(mpcpads)
midinotesmodel = modelutils.get_model_from_list(midinotes)

# views for drum editor
note_orders = {
    "chromatic" : midinotes,
    "mpc_banks_gm": mpc_banks_gm,
    "mpc_banks_chromatic" : mpc_banks_chromatic
    }

# general midi 1 drum map
gm1drumsmap = {
    35:"Acoustic Bass Drum",
    36:"Bass Drum 1",
    37:"Side Stick",
    38:"Acoustic Snare",
    39:"Hand Clap",
    40:"Electric Snare",
    41:"Low Floor Tom",
    42:"Closed Hi Hat",
    43:"High Floor Tom",
    44:"Pedal Hi-Hat",
    45:"Low Tom",
    46:"Open Hi-Hat",
    47:"Low-Mid Tom",
    48:"Hi-Mid Tom",
    49:"Crash Cymbal 1",
    50:"High Tom",
    51:"Ride Cymbal 1",
    52:"Chinese Cymbal",
    53:"Ride Bell",
    54:"Tambourine",
    55:"Splash Cymbal",
    56:"Cowbell",
    57:"Crash Cymbal 2",
    58:"Vibraslap",
    59:"Ride Cymbal 2",
    60:"Hi Bongo",
    61:"Low Bongo",
    62:"Mute Hi Conga",
    63:"Open Hi Conga",
    64:"Low Conga",
    65:"High Timbale",
    66:"Low Timbale",
    67:"High Agogo",
    68:"Low Agogo",
    69:"Cabasa",
    70:"Maracas",
    71:"Short Whistle",
    72:"Long Whistle",
    73:"Short Guiro",
    74:"Long Guiro",
    75:"Claves",
    76:"Hi Wood Block",
    77:"Low Wood Block",
    78:"Mute Cuica",
    79:"Open Cuica",
    80:"Mute Triangle",
    81:"Open Triangle"}

sampler_lists = {
    "filter" : ["Off", "2-Pole LP","2-Pole LP+","4-Pole LP","4-Pole LP+","6-Pole LP","2-Pole BP","2-Pole BP+","4-Pole BP","4-Pole BP+","6-Pole BP","1-Pole HP", "1-Pole HP+", "2-Pole HP", "2-Pole HP+", "6-Pole HP", "LO&lt;&gt;HI", "LO&lt;&gt;BAND", "BAND&lt;&gt;HI", "Notch 1", "Notch 2", "Notch 3", "Wide Notch", "Bi-Notch", "Peak 1", "Peak 2", "Peak 3", "Wide Peak", "Bi-Peak", "Phaser 1", "Phaser 2", "Bi-Phase", "Voweliser", "Triple"],
    "output" : ["L-R", "1-2", "3-4", "5-6", "7-8", "L", "R", "1", "2", "3", "4", "5", "6", "7", "8"],
    "midi_channel" : ["1A", "2A", "3A", "4A", "5A", "6A", "7A", "8A", "9A", "10A", "11A", "12A", "13A", "14A", "15A", "16A", "1B", "2B", "3B", "4B", "5B", "6B", "7B", "8B", "9B", "10B", "11B", "12B", "13B", "14B", "15B", "16B"],
    "filter_attenuation" : ["0db", "6db","12db","18db","24db","30db"],
    "fx_override" : ["Off", "A","B","C","D","AB","CD","Multi"],
    "zone_xfade" : ["Off", "Velocity", "Real-Time"],
    "zone_xfade_type" : ["Lin", "Exp", "Log"],
    "keygroup_edit_mode" : ["Single", "All","Add"],
    "keyboard_track" : ["Off", "On"],
    "playback" : ["1 Shot","LIR","LUR","LIR-&gt;Rt","Retrig","As Sample"],
    "playback_b" : ["Sample","1 Shot","No Loop","Loop"],
    "play_trigger" : ["Note On", "Note Off"],
    "glissando_mode" : ["Portamento", "Glissando"],
    "portamento_mode" : ["Time", "Rate"],
    "legato" : ["Off", "Pitch", "Loop"],
    "aftertouch_mode" : ["Channel", "Poly"],
    "pitchbend_mode" : ["Normal", "Held"],
    "program_type" : ["Keygroup", "Drum"],
    "keygroup_xfade_type" : ["Exp", "Log"],
    "reassignment_method" : ["Quietest", "Oldest"],
    "lfo_waves" : ["TRIANGLE", "SINE", "SQUARE", "SAW UP", "SAW DOWN", "RANDOM"],
    "lfo_retrigger" : ["Off", "On"],
    "lfo_sync" : ["Off", "On"],
    "lfo_midi_sync" : ["Off", "On"],
    "record_trigger_src" : ["OFF", "AUDIO", "MIDI"],
    "record_bit_depth" : ["16-Bit", "24-Bit"],
    "record_input" : ["ANALOGUE", "DIGITAL", "MAIN OUT", "ADAT 1/2", "ADAT 3/4", "ADAT 5/6", "ADAT 7/8"],
    "record_mode" : ["STEREO", "MONO L", "MONO R", "MONO L+R MIX"],
}



from aksy.devices.akai.filemodel import Chunk
import struct

def create_chunk(name, spec, *bytes):
    raw_bytes = struct.pack(spec, *bytes)
    return Chunk(name, raw_bytes, 
            len(raw_bytes)+8)

class ChunkBuilder:
    def build(self, chunk, fh):
        fh.write(struct.pack("<sl", chunk.name, chunk.length))
    
class ProgramWriter:
    def __init__(self):
        self.programBuilder = ProgramBuilder()
        
    def write(self, program, fh):
        fh.write(self.programBuilder.build(program))
        
class ProgramBuilder:
    def build(self, program):
        return struct.pack('<4sl8sl6b 4sl8b 4sl24b', 
            'RIFF', 
            0, 
            'APRG'
            'prg ', 
            6, 
            1, # set to zero program was not read...
            program.midi_prog_no, 
            len(program.keygroups), 
            0, 
            0, 
            0, 
            'out ', 
            8, 
            0, 
            program.loudness, 
            program.amp_mod1, 
            program.amp_mod2, 
            program.pan_mod1, 
            program.pan_mod2, 
            program.pan_mod3, 
            program.velo_sens, 
            'tune', 
            24, 
            0, 
            program.semi, 
            program.fine, 
            program.a_detune, 
            program.ais_detune, 
            program.b_detune, 
            program.bes_detune, 
            program.c_detune, 
            program.cis_detune, 
            program.des_detune, 
            program.e_detune, 
            program.f_detune, 
            program.fis_detune, 
            program.g_detune, 
            program.gis_detune, 
            program.pitchbend_up, 
            program.pitchbend_down, 
            program.bend_mode, 
            program.aftertouch, 
            0, 
            0, 
            0, 
            0, 
            0)

class ModulationBuilder:
    def build(self, modulation):
        bytes = (0, 
            0, 
            0, 
            0, 
            0, 
            modulation.amp_src1, 
            0, 
            modulation.amp_src2, 
            0, 
            modulation.pan_src1, 
            0, 
            modulation.pan_src2, 
            0, 
            modulation.pan_src3, 
            0, 
            modulation.lfo1_rate_src, 
            0, 
            modulation.lfo1_delay_src, 
            0, 
            modulation.lfo1_depth_src, 
            0, 
            modulation.lfo2_rate_src, 
            0, 
            modulation.lfo2_delay_src, 
            0, 
            modulation.lfo2_depth_src, 
            # keygroup mod
            0, 
            modulation.pitch1_src, 
            0, 
            modulation.pitch1_src, 
            0, 
            modulation.amp_src, 
            0, 
            modulation.filter_input1, 
            0, 
            modulation.filter_input2, 
            0, 
            modulation.filter_input3,)
        return create_chunk('mods', "%iB" % len(bytes))

class ZoneBuilder:
    def build(self, zone):
        return create_chunk('zone', '<2b20s22bh2b', 
            0, 
            len(zone.samplename), 
            zone.samplename, 
            0, 
            0, 
            0, 
            0, 
            0, 
            0, 
            0, 
            0, 
            0, 
            0, 
            0, 
            0, 
            zone.low_vel, 
            zone.high_vel, 
            zone.fine, 
            zone.semi, 
            zone.filter, 
            zone.pan, 
            zone.playback, 
            zone.output, 
            zone.level, 
            zone.kb_track, 
            zone.velo_start, 
            0, 
            0)



import samplerobject

class Multi(samplerobject.SamplerObject):
    def __init__(self,s,name,handle=None):
        samplerobject.SamplerObject.__init__(self,s,None,"multitools")
        
        self.name = name
        if handle:
            self.handle = handle
        else:
            self.handle = self.s.multitools.get_handle_by_name(self.name) 
            
        self.specialattrs = ["name",]

        self.attrs = ["no_parts"]
        
        if self.name:
            self.s.multitools.set_curr_by_name(self.name)
            self.update()
        else:
            print "No name..."
            
        self.precache()
            
    def set_name(self, name):
        self.s.multitools.rename_curr(name)
        
    def get_handle(self):
        # get handle by name or whatever, override in derived classes
        return self.handle

    def set_current(self):
        self.s.multitools.set_curr_by_name(self.name)



import os.path, os

def convert(filename):
	if os.path.exists(filename):
		[path, fileend] = os.path.split(filename)
		[name, ext] = os.path.splitext(fileend)
		src = filename
		dest = path + "\\" + name + "_rs" + ext
		cmd = "sox \"" + src + "\" -r 44100 \"" + dest + "\" resample -ql" 
		print cmd
		os.system(cmd)
		return dest
	else:
		return None
	# run cmd



from aksy.devices.akai import transfertools
from aksyx import AkaiSampler

from aksy import fileutils

from aksyosc.server import OSCServer
from aksyosc.handler import SamplerCallbackManager

import logging, asyncore

from threading import Thread

log = logging.getLogger("aksy")

class Sampler(object):
    """Base class for AkaiSampler.
    """
    MEMORY = AkaiSampler.MEMORY
    DISK = AkaiSampler.DISK
    def __init__(self, connector):
        self.connector = connector
        self.transfertools = transfertools.Transfertools(connector)

    def get_cmd_by_name(self, section_name, command_name):
        tools_obj = getattr(self, section_name)
        return getattr(tools_obj, command_name + "_cmd")

    def execute_by_cmd_name(self, section_name, command_name, args):
        tools_obj = getattr(self, section_name)
        func = getattr(tools_obj, command_name)
        return apply(func, args)

    def execute_alt_request(self, handle, commands, args, index = None):
        """Execute a list of commands on the item with the specified handle using Akai System Exclusive "Alternative Operations"
        All commands must be from the same sub section (get/set/main), the section id will be determined from the first command in the list.
        
        Examples:
        
            cmd = z48.sampletools.get_sample_length_cmd
            cmd2 = z48.sampletools.get_bit_depth_cmd
            cmd3 = z48.sampletools.get_playback_mode_cmd
            z48.execute_alt_request(65536, [cmd, cmd2, cmd3], [])
        (95955L, 16, 0)

            cmd = z48.sampletools.set_playback_mode_cmd
            cmd2 = z48.sampletools.set_orig_pitch_cmd
            z48.execute_alt_request(65536, [cmd, cmd2], [[1], [2]])

        """
        return self.connector.execute_alt_request(handle, commands, args, index)

    def close(self):
        self.connector.close()
        
    def start_osc_server(self):
        # for i in 'AK': print ord(i)
        OSCServer('localhost', 6575,  SamplerCallbackManager(self))
        class ServerThread(Thread):
            def __init__(self):
                Thread.__init__(self, name='OSC Server Thread')
            def run(self):
                asyncore.loop()
        self.serverThread = ServerThread()
        self.serverThread.start()

    def stop_osc_server(self):
        asyncore.close_all()
        self.serverThread.join()
        
    @staticmethod
    def is_filetype_supported(fname):
        return fileutils.is_file_type_supported(Sampler.get_supported_file_types(), fname)

    @staticmethod
    def get_supported_file_types():
        return ('wav', 'aif', 'aiff', 'akp', 'akm', 'mid', 'pgm')
    



#!/usr/bin/env python
import os,os.path,re,logging,sys,struct,math,traceback,getopt,inspect
import pygtk
pygtk.require('2.0')
import gtk,gtk.glade,gobject

# two panel browser

def get_selected_rows_from_treeview(treeview):
    """
    will return a single value or a list depending on what the selection mode is
    """
    if isinstance(treeview, gtk.TreeView):
        selection = treeview.get_selection()
    elif isinstance(treeview, gtk.TreeSelection):
        selection = treeview
    else:
        return None

    if selection.get_mode() == gtk.SELECTION_MULTIPLE:
        (model, pathlist) = selection.get_selected_rows()
        result = []
        for path in pathlist:
            result.append(model[path])
        return result
    else:
        (model, iter) = selection.get_selected()
        if iter:
            return model[iter] 
        else:
            # not sure why, but firing a second time
            return None


class Node:
    def __init__(self, path, parent_node = None):
        self.path = path 
        self.parent_node = parent_node 
        self.nodes = []

    def fullpath(self):
        return self.path

    def walk(self):
        print self.fullpath()
        for node in self.nodes:
            node.walk()

    def get_treestore(self, treestore = None, start_iter = None, file_list = False):
        if not treestore:
            treestore = gtk.TreeStore(str, str) 

        if (isinstance(self, DirectoryNode) and not file_list) or (isinstance(self, FileNode) and file_list): 
            node_iter = treestore.append(start_iter, [os.path.basename(self.path), self.path])
        else:
            node_iter = None

        # dont recurse for file list
        if not file_list or not start_iter:
            for node in self.nodes:
                node.get_treestore(treestore, node_iter, file_list)

        return treestore

    def get_files_treestore(self):
        return self.get_treestore(None, None, True)

class FileNode(Node):
    def __init__(self, path, parent_node = None):
        Node.__init__(self, path, parent_node)
        self.nodes = [] # file won't ever have this

class DirectoryNode(Node):
    def __init__(self, path, parent_node = None):
        Node.__init__(self, path, parent_node)
        self.nodes = self.get_list()

    def get_list(self):
        nodes = []
        paths = os.listdir(self.path)
        for path in paths:
            fullpath = self.path + "\\" + path
            if os.path.isdir(fullpath):
                node = DirectoryNode(fullpath, self)
            else:
                node = FileNode(fullpath, self)

            nodes.append(node)

        return nodes

class AkTreeView(gtk.TreeView):
    def __init__(self, treestore = None):
        gtk.TreeView.__init__(self, treestore)

        self.textedit = gtk.CellRendererText()
        self.textedit.set_property('editable', False)
        #textedit.connect('edited', self.on_textedit_changed, (treestore, 0))
        self.append_column(gtk.TreeViewColumn("Name", self.textedit, text=0))

class BrowserScrolledWindow(gtk.ScrolledWindow):
    def __init__(self, treestore = None):
        gtk.ScrolledWindow.__init__(self)
        self.treeview = AkTreeView(treestore)
        self.add(self.treeview)

class Browser(gtk.Window):
    def __init__(self, arguments):
        import getopt, glob, operator
        gtk.Window.__init__(self)
        #z48 = Devices.get_instance("z48", "usb")
        options, arguments = getopt.getopt(arguments, '?p')

        d = DirectoryNode("c:\\j\\mpc1000\\akpakm\\joe")
        self.hpaned = gtk.HPaned()
        self.treestore_dir = d.get_treestore()
        self.treestore_files = gtk.TreeStore(str, str) # temp
        # scrolling treeviews
        self.sw_dir = BrowserScrolledWindow(self.treestore_dir)
        self.sw_files = BrowserScrolledWindow(self.treestore_files)
        # hook up events
        self.sw_dir.treeview.connect("button-press-event", self.on_treeview_event)
        selection = self.sw_dir.treeview.get_selection()
        selection.connect("changed", self.on_dir_selection_changed)
        self.sw_files.treeview.connect("button-press-event", self.on_treeview_event)
        selection = self.sw_files.treeview.get_selection()
        selection.connect("changed", self.on_file_selection_changed)
        self.init_files(d.path)
        self.hpaned.add1(self.sw_dir)
        self.hpaned.add2(self.sw_files)
        self.add(self.hpaned)
        self.show_all()
        self.connect("delete-event", gtk.main_quit)

    def init_files(self, path):
        if not isinstance(path, DirectoryNode):
            d = DirectoryNode(path)
        else:
            d = path

        self.treestore_files = d.get_files_treestore()
        self.sw_files.treeview.set_model(self.treestore_files)
        self.sw_files.queue_draw()

    def on_treeview_event(self, widget, event):
        return False

    def on_dir_selection_changed(self, selection):
        selected_name = get_selected_rows_from_treeview(selection)
        self.init_files(selected_name[1]) # col 1 - full path

    def on_file_selection_changed(self, selection):
        treeview = selection.get_tree_view()
        selected_name = get_selected_rows_from_treeview(treeview)
        if selected_name:
            self.init_files(selected_name[1]) # col 1 - full path

def main(arguments):
    browser = Browser(arguments)
    gtk.main()


if __name__ == '__main__':
    main(sys.argv[1:])





""" Python equivalent of akai section songtools

Methods to manipulate in-memory songs
"""

__author__ =  'Walco van Loon'
__version__ =  '0.2'

from aksy.devices.akai.sysex import Command

import aksy.devices.akai.sysex_types

class Songtools:
    def __init__(self, s56k):
        self.sampler = s56k
        self.set_curr_by_name_cmd = Command('^', '\x16\x05', 'songtools', 'set_curr_by_name', (aksy.devices.akai.sysex_types.STRING,), (), aksy.devices.akai.sysex_types.S56K_USERREF)
        self.set_curr_by_handle_cmd = Command('^', '\x16\x06', 'songtools', 'set_curr_by_handle', (aksy.devices.akai.sysex_types.CWORD,), (), aksy.devices.akai.sysex_types.S56K_USERREF)
        self.delete_curr_cmd = Command('^', '\x16\x08', 'songtools', 'delete_curr', (), (), aksy.devices.akai.sysex_types.S56K_USERREF)
        self.rename_cmd = Command('^', '\x16\x09', 'songtools', 'rename', (), (aksy.devices.akai.sysex_types.STRING,), aksy.devices.akai.sysex_types.S56K_USERREF)
        self.get_no_items_cmd = Command('^', '\x16\x10', 'songtools', 'get_no_items', (), (aksy.devices.akai.sysex_types.CWORD,), aksy.devices.akai.sysex_types.S56K_USERREF)
        self.get_name_by_handle_cmd = Command('^', '\x16\x11', 'songtools', 'get_name_by_handle', (aksy.devices.akai.sysex_types.CWORD,), (aksy.devices.akai.sysex_types.STRINGARRAY,), aksy.devices.akai.sysex_types.S56K_USERREF)
        self.get_curr_handle_cmd = Command('^', '\x16\x13', 'songtools', 'get_curr_handle', (), (aksy.devices.akai.sysex_types.CWORD,), aksy.devices.akai.sysex_types.S56K_USERREF)
        self.get_curr_name_cmd = Command('^', '\x16\x14', 'songtools', 'get_curr_name', (), (aksy.devices.akai.sysex_types.STRINGARRAY,), aksy.devices.akai.sysex_types.S56K_USERREF)

    def rename(self):
        """Rename Song
        """
        return self.sampler.execute(self.rename_cmd, ())

    def get_no_items(self):
        """Get Number of Songs in Memory

        Returns:
            WORD
        """
        return self.sampler.execute(self.get_no_items_cmd, ())

    def get_curr_handle(self):
        """Current Song Index (i.e., its position in memory)

        Returns:
            WORD
        """
        return self.sampler.execute(self.get_curr_handle_cmd, ())

    def get_curr_name(self):
        """Get Current Song Name

        Returns:
            STRING
        """
        return self.sampler.execute(self.get_curr_name_cmd, ())

    def get_names(self):
        """Get the Names of All Songs in memory

        Returns:
            STRINGARRAY
        """
        names = []
        count = self.get_no_items()
        for i in range(count):
            self.set_curr_by_handle(i)
            names.extend(self.get_curr_name())
        return names

    def set_curr_by_handle(self, arg0):
        """Select by handle
        """
        return self.sampler.execute(self.set_curr_by_handle_cmd, (arg0, ))

    def set_curr_by_name(self, arg0):
        """Select by name
        """
        return self.sampler.execute(self.set_curr_by_name_cmd, (arg0, ))

    def delete_curr(self):
        """Delete current item from memory
        """
        return self.sampler.execute(self.delete_curr_cmd, ())

    def get_name_by_handle(self, arg0):
        """Get item name from handle

        Returns:
            STRING
        """
        return self.sampler.execute(self.get_name_by_handle_cmd, (arg0, ))

    def get_handles_names(self):
        """Get list of song handles and names

        Returns:
            HANDLENAMEARRAY
        """
        handle_names = []
        count = self.get_no_items()
        for i in range(count):
            handle_names.extend([i, self.get_curr_name()])
        return handle_names



#!/usr/bin/env python
import gtk

import os.path

# our stuff
from aksui.utils import midiutils, modelutils
from aksui.ak import envelope, keygroup, part

import rangewidget

__author__ = 'Joseph Misra'
__version__ = '0.71'

class EnvelopeHBox(gtk.HBox):
    def __init__(self, kg, index):
        gtk.HBox.__init__(self)
        self.s = kg.s
        self.setup(kg, index)

    def setup(self, kg, index):
        self.e = envelope.Envelope(kg, index)
        
        self.clear_widgets()
        
        if index == 0:
            # amp envelope is simpler
            knobs = ['rate1','rate2','level2','rate3']
            self.e.abbr['envelope_rate1'] = 'A'
            self.e.abbr['envelope_rate2'] = 'D'
            self.e.abbr['envelope_level2'] = 'S'
            self.e.abbr['envelope_rate3'] = 'R'
        else:
            knobs = ['rate1','level1','rate2','level2','rate3','level3','rate4','level4','reference']
            
        for knob_name in knobs:
            knob = rangewidget.AkKnobWidget(self.e, 'envelope_' + knob_name, 0, 100, 1, None)
            self.pack_start(knob, False, False, 2)
            
    def clear_widgets(self):
        for child in self.get_children():
            self.remove(child)
            
class MultiEditorVBox(gtk.VBox):
    """
    Minimal multi editor VBox
    """
    def __init__(self, s, m):
        gtk.VBox.__init__(self)
        self.s = s
        #self.connect("delete-event", self.on_delete_event)
        self.setup(m)
    
    def setup(self, m):
        self.m = m
        
        self.no_parts = self.m.no_parts
        
        self.clear_widgets()
        
        for i in range(self.no_parts):
            p = part.Part(self.s, m, i)
        
            kghboxall = gtk.HBox()
            
            kghboxall.pack_start(rangewidget.AkComboBox(p, "multi_part_name", self.s.programsmodel, False),True,True,2)
            kghboxall.pack_start(rangewidget.PartRangeWidget(p, "part_level"), True, True, 2)
            kghboxall.pack_start(rangewidget.AkKnobWidget(p, "part_pan", 0, 100, 1, None), False, False, 2)
            kghboxall.pack_start(rangewidget.AkComboBox(p, "part_output", midiutils.sampler_lists["output"], True),True,True,2)
            kghboxall.pack_start(rangewidget.AkComboBox(p, "part_midi_channel", midiutils.sampler_lists["midi_channel"], True),True,True,2)
            
            self.pack_start(kghboxall, False, False, 2)

    def clear_widgets(self):
        for child in self.get_children():
            self.remove(child)

class DrumEditorTable(gtk.Table):
    def __init__(self, s, p):
        self.dnd_list = [ ( 'text/uri-list', 0, 80 ) ] 
        self.columns = 4
        self.note_order = midiutils.note_orders["mpc_banks_gm"] # chromatic / mpc_banks_gm / mpc_banks_chromatic
        gtk.Table.__init__(self, (len(self.note_order) / self.columns), self.columns, True) # 32 rows, 4 columns, homogenous
        self.s = s
        self.on_toggled_callback = None
        
        self.setup(p)

    def on_drag_data_received(self, widget, context, x, y, selection, target_type, timestamp, kg, cb, zone_index):
        self.s.FileChooser.on_drag_data_received(widget, context, x, y, selection, target_type, timestamp)
        # if it was a single selection, set that zone
        if len(self.s.FileChooser.files) > 0 and kg:
            first_file = os.path.basename(self.s.FileChooser.files[0])
            (filename, ext) = first_file.split(".")
            kg.zones[zone_index].set("sample", filename)
        # hack, this should be updating rather than completely refreshing
        cb.set_model(self.s.samplesmodel)
        cb.somodel = self.s.samplesmodel
        cb.init()
        
    def setup(self, p):
        self.p = p
        self.no_keygroups = self.p.no_keygroups
        self.clear_widgets()
        rbg = None # radio button group
       
        self.zone_labels = {}
        for row in range(0,(len(self.note_order) / self.columns)):
            for column in range(0,self.columns): 
                index = (row * self.columns) + column
                if index < len(self.note_order):
                    i = self.note_order[index]
                    kg = keygroup.get_keygroup_cached(p, i)
                    desc = midiutils.midinotes[i]
                    tb = gtk.RadioButton(rbg)
                    tb.connect("toggled", self.on_button_press_event, kg.index + 1) 

                    if i in midiutils.gm1drumsmap.keys():
                        subdesc = midiutils.gm1drumsmap[i]
                        # general midi markup label
                        subdesclabel = gtk.Label("<span size='smaller'>%s %s</span>" % (desc, subdesc))
                        subdesclabel.set_use_markup(True)
                    else:
                        subdesclabel = gtk.Label("<span size='smaller'>%s</span>" % (desc))
                        subdesclabel.set_use_markup(True)
                    
                    if not rbg:
                        rbg = tb
                        tb.set_active(True)
                    
                    vboxall = gtk.VBox()
                    kghboxall = gtk.HBox()
                    kghboxall.pack_start(tb, False, False, 1)
                    if subdesclabel:
                        kghboxall.pack_start(subdesclabel, False, False, 0)
                    #Dunno if these are really necessary - removing to save space.
                    #kghboxall.pack_start(rangewidget.AkKnobWidget(kg, "level", -600, 60, 10, "db"), False, False, 2)
                    #kghboxall.pack_start(rangewidget.AkKnobWidget(kg, "tune", -3600, 3600, 100, ""), False, False, 2)
                    vboxall.pack_start(kghboxall, False, False, 1)
                    eventbox = gtk.EventBox()
                    eventbox.connect("button_press_event", self.on_label_click, tb)
                    eventbox.set_events(gtk.gdk.BUTTON_PRESS_MASK)
                    zone_vbox = gtk.VBox(False, 0)
                    self.zone_labels[i] = []
                    for zone_index in range(4):
                        zone_label = rangewidget.AkLabel(kg.zones[zone_index], "sample", self.s.samplesmodel, False)
                        zone_label.set_justify(gtk.JUSTIFY_LEFT)
                        self.zone_labels[i].append(zone_label)
                        zone_label.drag_dest_set(gtk.DEST_DEFAULT_MOTION | gtk.DEST_DEFAULT_HIGHLIGHT | gtk.DEST_DEFAULT_DROP, self.dnd_list, gtk.gdk.ACTION_COPY)
                        zone_label.connect("drag_data_received", self.on_drag_data_received, kg, zone_label, zone_index)
                        zone_vbox.pack_start(zone_label, False, False, 0)
                    vboxall.pack_start(zone_vbox, False, False, 0)
                    eventbox.add(vboxall)
                    self.attach(eventbox,column,column+1,row,row+1)

    def clear_widgets(self):
        for child in self.get_children():
            self.remove(child)

    def on_label_click(self, widget, e, toggle):
        toggle.set_active(True)

    def on_button_press_event(self, widget, data = None):
        if self.on_toggled_callback:
            self.on_toggled_callback(widget, data)

class KeygroupEditorVBox(gtk.VBox):
    """
    Minimal keygroup editor VBox
    """
    def __init__(self, s, p):
        gtk.VBox.__init__(self)
        self.s = s
        #self.connect("delete-event", self.on_delete_event)
        self.setup(p)
        self.on_toggled_callback = None

    def setup(self, p):
        self.p = p
        
        self.no_keygroups = self.p.no_keygroups
        
        self.clear_widgets()
        rbg = None # radio button group

        for i in range(self.no_keygroups):
            kg = keygroup.get_keygroup_cached(p, i)
            
            # TODO: Switch to a radio button.
            tb = gtk.RadioButton(rbg, str(i + 1))
            tb.connect("toggled", self.on_button_press_event, (i + 1))
            
            if not rbg:
                rbg = tb
                tb.set_active(True)
            
            kghboxall = gtk.HBox()
            
            kghboxall.pack_start(tb, False, False, 1)
            kghboxall.pack_start(rangewidget.KeygroupRangeWidget(kg))
            #kghboxall.pack_start(rangewidget.AkComboBox(kg.zones[0], "sample", self.s.samplesmodel, False))
            kghboxall.pack_start(rangewidget.AkKnobWidget(kg, "level", -600, 60, 10, "db"), False, False, 2)
            kghboxall.pack_start(rangewidget.AkKnobWidget(kg, "tune", -3600, 3600, 100, ""), False, False, 2)
            self.pack_start(kghboxall, False, False, 2)
            
    def clear_widgets(self):
        for child in self.get_children():
            self.remove(child)

    def on_button_press_event(self, widget, data = None):
        if self.on_toggled_callback:
            self.on_toggled_callback(widget, data)
